@@ -12,7 +12,7 @@
 #include "wocky-test-helper.h"
 
 static void
-test_build_iq_result (void)
+test_build_iq_result_simple_ack (void)
 {
   WockyStanza *iq, *reply, *expected;
 
@@ -37,6 +37,21 @@ test_build_iq_result (void)
 
   g_object_unref (reply);
   g_object_unref (expected);
+  g_object_unref (iq);
+}
+
+static void
+test_build_iq_result_complex_reply (void)
+{
+  WockyStanza *iq, *reply, *expected;
+
+  iq = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+    WOCKY_STANZA_SUB_TYPE_GET, "juliet@example.com", "romeo@example.net",
+    '@', "id", "one",
+      '(', "query",
+        ':', "http://jabber.org/protocol/disco#items",
+      ')',
+    NULL);
 
   /* Send a more complex reply */
   expected = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
@@ -67,6 +82,12 @@ test_build_iq_result (void)
   g_object_unref (reply);
   g_object_unref (expected);
   g_object_unref (iq);
+}
+
+static void
+test_build_iq_result_no_to_attr (void)
+{
+  WockyStanza *iq, *reply, *expected;
 
   /* Send a reply to an IQ with no "to" attribute. */
   iq = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
@@ -94,7 +115,7 @@ test_build_iq_result (void)
 }
 
 static void
-test_build_iq_error (void)
+test_build_iq_error_simple_error (void)
 {
   WockyStanza *iq, *reply, *expected;
 
@@ -110,6 +131,9 @@ test_build_iq_error (void)
   expected = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
     WOCKY_STANZA_SUB_TYPE_ERROR, "romeo@example.net", "juliet@example.com",
     '@', "id", "one",
+    '(', "query",
+      ':', "http://jabber.org/protocol/disco#items",
+    ')',
     NULL);
 
   reply = wocky_stanza_build_iq_error (iq, NULL);
@@ -119,6 +143,21 @@ test_build_iq_error (void)
 
   g_object_unref (reply);
   g_object_unref (expected);
+  g_object_unref (iq);
+}
+
+static void
+test_build_iq_error_complex (void)
+{
+  WockyStanza *iq, *reply, *expected;
+
+  iq = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+    WOCKY_STANZA_SUB_TYPE_GET, "juliet@example.com", "romeo@example.net",
+    '@', "id", "one",
+      '(', "query",
+        ':', "http://jabber.org/protocol/disco#items",
+      ')',
+    NULL);
 
   /* Send a more complex reply */
   expected = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
@@ -126,20 +165,17 @@ test_build_iq_error (void)
     '@', "id", "one",
       '(', "query",
         ':', "http://jabber.org/protocol/disco#items",
-        '(', "error",
-          '@', "code", "403",
-          '@', "type", "auth",
-        ')',
+      ')',
+      '(', "error",
+        '@', "code", "403",
+        '@', "type", "auth",
       ')',
     NULL);
 
   reply = wocky_stanza_build_iq_error (iq,
-      '(', "query",
-        ':', "http://jabber.org/protocol/disco#items",
-        '(', "error",
-          '@', "code", "403",
-          '@', "type", "auth",
-        ')',
+      '(', "error",
+        '@', "code", "403",
+        '@', "type", "auth",
       ')',
       NULL);
 
@@ -227,10 +263,9 @@ test_extract_stanza_error (void)
 }
 
 static void
-test_extract_errors (void)
+test_extract_errors_not_error (void)
 {
   WockyStanza *stanza;
-  const gchar *description = "I am a sentence.";
   WockyXmppErrorType type;
   GError *core = NULL, *specialized = NULL;
   WockyNode *specialized_node = NULL;
@@ -253,6 +288,16 @@ test_extract_errors (void)
   g_assert (specialized_node == NULL);
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_without_description (void)
+{
+  WockyStanza *stanza;
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
+  gboolean ret;
 
   /* Test a boring error with no description */
   stanza = wocky_stanza_build (
@@ -280,6 +325,16 @@ test_extract_errors (void)
   g_assert (specialized_node == NULL);
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_with_text (void)
+{
+  WockyStanza *stanza;
+  const gchar *description = "I am a sentence.";
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
 
   /* Now a different error with some text */
   stanza = wocky_stanza_build (
@@ -310,6 +365,15 @@ test_extract_errors (void)
   g_assert (specialized_node == NULL);
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_application_specific_unknown (void)
+{
+  WockyStanza *stanza;
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
 
   /* Another error, with an application-specific element we don't understand */
   stanza = wocky_stanza_build (
@@ -338,14 +402,26 @@ test_extract_errors (void)
 
   g_assert_no_error (specialized);
 
-  /* This is questionable: maybe wocky_xmpp_error_extract() should assume that
-   * a child of <error/> in a NS it doesn't understand is a specialized error,
-   * rather than requiring the ns to be registered with
-   * wocky_xmpp_error_register_domain().
+  /* The namespace is not registered, @specialized is not set. However we
+   * assume that any other namespace element is a specialized error, and it
+   * should get returned in @specialized_node
    */
-  g_assert (specialized_node == NULL);
+  g_assert (specialized_node);
+  g_assert_cmpstr (specialized_node->name, ==, "buy-a-private-cloud");
+  g_assert_cmpstr (wocky_node_get_ns (specialized_node), ==,
+      "http://example.com/angry-cloud");
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_jingle_error (void)
+{
+  WockyStanza *stanza;
+  const gchar *description = "I am a sentence.";
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
 
   /* A Jingle error! With the child nodes in an erratic order */
   stanza = wocky_stanza_build (
@@ -387,6 +463,66 @@ test_extract_errors (void)
   wocky_stanza_extract_errors (stanza, NULL, NULL, NULL, NULL);
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_extra_application_specific (void)
+{
+  WockyStanza *stanza;
+  const gchar *description = "I am a sentence.";
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
+
+  /* Jingle error! + Bogus extra app specific error, which should be ignored */
+  stanza = wocky_stanza_build (
+      WOCKY_STANZA_TYPE_IQ, WOCKY_STANZA_SUB_TYPE_ERROR,
+      "from", "to",
+        '(', "error",
+          '@', "type", "cancel",
+          '(', "tie-break",
+            ':', WOCKY_XMPP_NS_JINGLE_ERRORS,
+          ')',
+          '(', "out-of-order",
+            ':', WOCKY_XMPP_NS_JINGLE_ERRORS,
+          ')',
+          '(', "text",
+            ':', WOCKY_XMPP_NS_STANZAS,
+            '$', description,
+          ')',
+          '(', "conflict",
+            ':', WOCKY_XMPP_NS_STANZAS,
+          ')',
+        ')',
+      NULL);
+
+  wocky_stanza_extract_errors (stanza, &type, &core, &specialized,
+      &specialized_node);
+
+  g_assert_cmpuint (type, ==, WOCKY_XMPP_ERROR_TYPE_CANCEL);
+
+  g_assert_error (core, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_CONFLICT);
+  g_assert_cmpstr (core->message, ==, description);
+  g_clear_error (&core);
+
+  g_assert_error (specialized, WOCKY_JINGLE_ERROR,
+      WOCKY_JINGLE_ERROR_TIE_BREAK);
+  g_assert_cmpstr (specialized->message, ==, description);
+  g_clear_error (&specialized);
+
+  g_assert (specialized_node != NULL);
+  g_assert_cmpstr (specialized_node->name, ==, "tie-break");
+
+  g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_legacy_code (void)
+{
+  WockyStanza *stanza;
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
 
   /* How about a legacy error code? */
   stanza = wocky_stanza_build (
@@ -414,6 +550,15 @@ test_extract_errors (void)
   g_assert (specialized_node == NULL);
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_no_sense (void)
+{
+  WockyStanza *stanza;
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
 
   /* An error that makes no sense */
   stanza = wocky_stanza_build (
@@ -439,9 +584,19 @@ test_extract_errors (void)
   g_clear_error (&core);
 
   g_assert_no_error (specialized);
-  g_assert (specialized_node == NULL);
+  g_assert (specialized_node != NULL);
+  g_assert_cmpstr (specialized_node->name, ==, "hoobily-lala-whee");
 
   g_object_unref (stanza);
+}
+
+static void
+test_extract_errors_not_really (void)
+{
+  WockyStanza *stanza;
+  WockyXmppErrorType type;
+  GError *core = NULL, *specialized = NULL;
+  WockyNode *specialized_node = NULL;
 
   /* And finally, a stanza with type='error' but no <error/> at all... */
   stanza = wocky_stanza_build (
@@ -469,7 +624,7 @@ test_extract_errors (void)
   } G_STMT_END
 
 static void
-test_stanza_error_to_node (void)
+test_stanza_error_to_node_core (void)
 {
   GError *e = NULL;
   GError *core = NULL, *specialized = NULL;
@@ -514,6 +669,15 @@ test_stanza_error_to_node (void)
   g_object_unref (expected);
   g_clear_error (&e);
   g_clear_error (&core);
+}
+
+static void
+test_stanza_error_to_node_jingle (void)
+{
+  GError *e = NULL;
+  GError *core = NULL, *specialized = NULL;
+  const gchar *description = "bzzzt";
+  WockyStanza *stanza, *expected;
 
   /* How about a nice game of Jingle? */
   g_set_error_literal (&e, WOCKY_JINGLE_ERROR,
@@ -565,13 +729,40 @@ main (int argc, char **argv)
   int result;
 
   test_init (argc, argv);
-  g_test_add_func ("/xmpp-stanza/build-iq-result", test_build_iq_result);
-  g_test_add_func ("/xmpp-stanza/build-iq-error", test_build_iq_error);
-  g_test_add_func ("/xmpp-stanza/extract-stanza-error",
+  g_test_add_func ("/xmpp-stanza/iq-result/build-simple-ack",
+      test_build_iq_result_simple_ack);
+  g_test_add_func ("/xmpp-stanza/iq-result/build-complex-reply",
+      test_build_iq_result_complex_reply);
+  g_test_add_func ("/xmpp-stanza/iq-result/build-no-to-attr",
+      test_build_iq_result_no_to_attr);
+  g_test_add_func ("/xmpp-stanza/errors/build-simple",
+      test_build_iq_error_simple_error);
+  g_test_add_func ("/xmpp-stanza/errors/build-complex",
+      test_build_iq_error_complex);
+  g_test_add_func ("/xmpp-stanza/errors/extract-stanza",
       test_extract_stanza_error);
-  g_test_add_func ("/xmpp-stanza/extract-errors", test_extract_errors);
-  g_test_add_func ("/xmpp-stanza/stanza-error-to-node",
-      test_stanza_error_to_node);
+  g_test_add_func ("/xmpp-stanza/errors/not-error",
+      test_extract_errors_not_error);
+  g_test_add_func ("/xmpp-stanza/errors/without-description",
+      test_extract_errors_without_description);
+  g_test_add_func ("/xmpp-stanza/errors/with-text",
+      test_extract_errors_with_text);
+  g_test_add_func ("/xmpp-stanza/errors/application-specific-unknown",
+      test_extract_errors_application_specific_unknown);
+  g_test_add_func ("/xmpp-stanza/errors/jingle-error",
+      test_extract_errors_jingle_error);
+  g_test_add_func ("/xmpp-stanza/errors/extra-application-specific",
+      test_extract_errors_extra_application_specific);
+  g_test_add_func ("/xmpp-stanza/errors/legacy-code",
+      test_extract_errors_legacy_code);
+  g_test_add_func ("/xmpp-stanza/errors/no-sense",
+      test_extract_errors_no_sense);
+  g_test_add_func ("/xmpp-stanza/errors/not-really",
+      test_extract_errors_not_really);
+  g_test_add_func ("/xmpp-stanza/errors/stanza-to-node",
+      test_stanza_error_to_node_core);
+  g_test_add_func ("/xmpp-stanza/errors/stanza-to-node-jingle",
+      test_stanza_error_to_node_jingle);
 
   result =  g_test_run ();
   test_deinit ();
