@@ -30,6 +30,7 @@
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/interfaces.h>
 
+#include <wocky/wocky-c2s-porter.h>
 #include <wocky/wocky-utils.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_CONNECTION
@@ -42,13 +43,17 @@
 #include "presence-cache.h"
 #include "presence.h"
 #include "roster.h"
+#include "conn-util.h"
 #include "util.h"
 
+#define GOOGLE_SHARED_STATUS_VERSION "2"
+
 typedef enum {
     INVISIBILITY_METHOD_NONE = 0,
     INVISIBILITY_METHOD_PRESENCE_INVISIBLE, /* presence type=invisible */
     INVISIBILITY_METHOD_PRIVACY,
-    INVISIBILITY_METHOD_INVISIBLE_COMMAND
+    INVISIBILITY_METHOD_INVISIBLE_COMMAND,
+    INVISIBILITY_METHOD_SHARED_STATUS
 } InvisibilityMethod;
 
 struct _GabbleConnectionPresencePrivate {
@@ -56,6 +61,9 @@ struct _GabbleConnectionPresencePrivate {
     LmMessageHandler *iq_list_push_cb;
     gchar *invisible_list_name;
 
+    /* Mapping between status "show" strings, and shared statuses */
+    GHashTable *shared_statuses;
+
     /* Map of presence statuses backed by privacy lists. This
      * will be NULL until we receive a (possibly empty) list of
      * all lists in get_existing_privacy_lists_cb().
@@ -63,6 +71,15 @@ struct _GabbleConnectionPresencePrivate {
      * gchar *presence_status_name → gchar *privacy_list
      */
     GHashTable *privacy_statuses;
+
+    /* Are all of the connected resources complying to version 2 */
+    gboolean shared_status_compat;
+
+    /* Max statuses in a shared status list */
+    gint max_shared_statuses;
+
+    /* The shared status IQ handler */
+    guint iq_shared_status_cb;
 };
 
 static const TpPresenceStatusOptionalArgumentSpec gabble_status_arguments[] = {
@@ -74,7 +91,7 @@ static const TpPresenceStatusOptionalArgumentSpec gabble_status_arguments[] = {
 
 /* order must match PresenceId enum in connection.h */
 /* in increasing order of presence */
-static const TpPresenceStatusSpec base_statuses[] = {
+static const TpPresenceStatusSpec gabble_base_statuses[] = {
   { "offline", TP_CONNECTION_PRESENCE_TYPE_OFFLINE, FALSE,
     gabble_status_arguments, NULL, NULL },
   { "unknown", TP_CONNECTION_PRESENCE_TYPE_UNKNOWN, FALSE,
@@ -130,6 +147,9 @@ static void toggle_initial_presence_visibility_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data);
 
+static void activate_current_privacy_list (GabbleConnection *self,
+    GSimpleAsyncResult *result);
+
 /* actual code! */
 
 GQuark
@@ -181,8 +201,7 @@ construct_contact_statuses_cb (GObject *obj,
         }
       else
         {
-         if (gabble_roster_handle_get_subscription (self->roster, handle)
-             & GABBLE_ROSTER_SUBSCRIPTION_TO)
+         if (gabble_roster_handle_sends_presence_to_us (self->roster, handle))
            status = GABBLE_PRESENCE_OFFLINE;
          else
            status = GABBLE_PRESENCE_UNKNOWN;
@@ -251,12 +270,160 @@ emit_presences_changed_for_self (GabbleConnection *self)
   g_array_free (handles, TRUE);
 }
 
+static WockyStanza *
+build_shared_status_stanza (GabbleConnection *self)
+{
+  GabblePresence *presence = self->self_presence;
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  const gchar *bare_jid = conn_util_get_bare_self_jid (self);
+  gpointer key, value;
+  GHashTableIter iter;
+  WockyNode *query_node = NULL;
+  WockyStanza *iq = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+      WOCKY_STANZA_SUB_TYPE_SET, NULL, bare_jid,
+        '(', "query",
+          ':', NS_GOOGLE_SHARED_STATUS,
+          '*', &query_node,
+          '@', "version", GOOGLE_SHARED_STATUS_VERSION,
+          '(', "status",
+            '$', presence->status_message,
+          ')',
+          '(', "show",
+            '$', presence->status == GABBLE_PRESENCE_DND ? "dnd" : "default",
+          ')',
+        ')',
+      NULL);
+
+  g_hash_table_iter_init (&iter, priv->shared_statuses);
+
+  while (g_hash_table_iter_next (&iter, &key, &value))
+    {
+      gchar **status_iter;
+      gchar **statuses = (gchar **) value;
+      WockyNode *list_node = wocky_node_add_child (query_node, "status-list");
+
+      wocky_node_set_attribute (list_node, "show", (const gchar *) key);
+
+      for (status_iter = statuses; *status_iter != NULL; status_iter++)
+        wocky_node_add_child_with_content (list_node, "status", *status_iter);
+    }
+
+  wocky_node_set_attribute (wocky_node_add_child (query_node, "invisible"), "value",
+      presence->status == GABBLE_PRESENCE_HIDDEN ? "true" : "false");
+
+  return iq;
+}
+
+static void
+set_shared_status_cb (GObject *source_object,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (user_data);
+  GabbleConnection *self = GABBLE_CONNECTION (source_object);
+  GError *error = NULL;
+
+  if (!conn_util_send_iq_finish (self, res, NULL, &error) ||
+      !conn_presence_signal_own_presence (self, NULL, &error))
+    {
+      g_simple_async_result_set_error (result,
+          CONN_PRESENCE_ERROR, CONN_PRESENCE_ERROR_SET_SHARED_STATUS,
+          "error setting Google shared status: %s", error->message);
+    }
+
+      g_simple_async_result_complete (result);
+      g_object_unref (result);
+
+  if (error != NULL)
+    g_error_free (error);
+}
+
+static void
+insert_presence_to_shared_statuses (GabbleConnection *self)
+{
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  GabblePresence *presence = self->self_presence;
+  const gchar *show = presence->status == GABBLE_PRESENCE_DND ? "dnd" : "default";
+  gchar **statuses = g_hash_table_lookup (priv->shared_statuses, show);
+
+  if (presence->status_message == NULL)
+    return;
+
+  if (statuses == NULL)
+    {
+      statuses = g_new0 (gchar *, 2);
+      statuses[0] = g_strdup (presence->status_message);
+      g_hash_table_insert (priv->shared_statuses, g_strdup (show), statuses);
+    }
+  else
+    {
+      guint i;
+      guint list_len = MIN (priv->max_shared_statuses,
+          (gint) g_strv_length (statuses) + 1);
+      gchar **new_statuses = g_new0 (gchar *, list_len + 1);
+
+      new_statuses[0] = g_strdup (presence->status_message);
+
+      for (i = 1; i < list_len; i++)
+        new_statuses[i] = g_strdup (statuses[i - 1]);
+
+      g_hash_table_insert (priv->shared_statuses, g_strdup (show), new_statuses);
+    }
+}
+
+static void
+set_shared_status (GabbleConnection *self,
+    GSimpleAsyncResult *result)
+{
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  GabblePresence *presence = self->self_presence;
+  WockyStanza *iq;
+
+  g_object_ref (result);
+
+  DEBUG ("shared status invisibility is %savailable",
+      priv->shared_status_compat ? "" : "un");
+
+  if (presence->status == GABBLE_PRESENCE_HIDDEN && !priv->shared_status_compat)
+    presence->status = GABBLE_PRESENCE_DND;
+
+  insert_presence_to_shared_statuses (self);
+
+  iq = build_shared_status_stanza (self);
+
+  conn_util_send_iq_async (self, iq, NULL, set_shared_status_cb, result);
+
+  g_object_unref (iq);
+}
+
+static void
+set_shared_status_async (GabbleConnection *self,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result = g_simple_async_result_new (G_OBJECT (self),
+      callback, user_data, set_shared_status_async);
+
+  set_shared_status (self, result);
+
+  g_object_unref (result);
+}
+
+static gboolean
+set_shared_status_finish (GabbleConnection *self,
+    GAsyncResult *result,
+    GError **error)
+{
+  wocky_implement_finish_void (self, set_shared_status_async);
+}
+
 static void
 set_xep0186_invisible (GabbleConnection *self,
     gboolean invisible,
     GSimpleAsyncResult *result)
 {
   TpBaseConnection *base = (TpBaseConnection *) self;
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
   GError *error = NULL;
   const gchar *element = invisible ? "invisible" : "visible";
   WockyStanza *iq = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
@@ -268,9 +435,18 @@ set_xep0186_invisible (GabbleConnection *self,
 
   if (!invisible && base->status != TP_CONNECTION_STATUS_CONNECTED)
     {
-      conn_presence_signal_own_presence (self, NULL, &error);
-      g_simple_async_result_complete_in_idle (result);
-      g_object_unref (result);
+      if (priv->privacy_statuses != NULL)
+        {
+          /* A plugin might need to activate a privacy list */
+          activate_current_privacy_list (self, result);
+          g_object_unref (result);
+        }
+      else
+        {
+          conn_presence_signal_own_presence (self, NULL, &error);
+          g_simple_async_result_complete_in_idle (result);
+          g_object_unref (result);
+        }
     }
   else if (!_gabble_connection_send_with_reply (self, (LmMessage *) iq,
           set_xep0186_invisible_cb, NULL, result, &error))
@@ -443,14 +619,17 @@ disable_invisible_privacy_list (GabbleConnection *self)
 {
   GabbleConnectionPresencePrivate *priv = self->presence_priv;
 
-  if (self->features & GABBLE_CONNECTION_FEATURES_PRESENCE_INVISIBLE)
-    priv->invisibility_method = INVISIBILITY_METHOD_PRESENCE_INVISIBLE;
-  else
-    priv->invisibility_method = INVISIBILITY_METHOD_NONE;
+  if (priv->invisibility_method == INVISIBILITY_METHOD_PRIVACY)
+    {
+      if (self->features & GABBLE_CONNECTION_FEATURES_PRESENCE_INVISIBLE)
+        priv->invisibility_method = INVISIBILITY_METHOD_PRESENCE_INVISIBLE;
+      else
+        priv->invisibility_method = INVISIBILITY_METHOD_NONE;
 
-  DEBUG ("Set invisibility method to %s",
-      (priv->invisibility_method == INVISIBILITY_METHOD_PRESENCE_INVISIBLE) ?
-      "presence-invisible" : "none");
+      DEBUG ("Set invisibility method to %s",
+          (priv->invisibility_method == INVISIBILITY_METHOD_PRESENCE_INVISIBLE)
+          ? "presence-invisible" : "none");
+    }
 }
 
 static LmHandlerResult
@@ -524,7 +703,8 @@ create_invisible_privacy_list_finish (GabbleConnection *self,
   wocky_implement_finish_void (self, create_invisible_privacy_list_async);
 }
 
-static void create_invisible_privacy_list_cb (GObject *source_object,
+static void
+create_invisible_privacy_list_cb (GObject *source_object,
     GAsyncResult *result,
     gpointer user_data)
 {
@@ -552,6 +732,129 @@ static void create_invisible_privacy_list_cb (GObject *source_object,
       toggle_initial_presence_visibility_cb, user_data);
 }
 
+static gboolean
+store_shared_statuses (GabbleConnection *self,
+    WockyNode *query_node)
+{
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  TpBaseConnection *base = TP_BASE_CONNECTION (self);
+  GabblePresenceId presence_id = self->self_presence->status;
+  const gchar *status_message = NULL;
+  const gchar *min_version = wocky_node_get_attribute (query_node, "status-min-ver");
+  WockyNodeIter iter;
+  gchar *resource;
+  guint8 prio;
+  gboolean rv;
+  WockyNode *node;
+  gboolean dnd = FALSE;
+  gboolean invisible = FALSE;
+
+  DEBUG ("status-min-ver %s", min_version);
+
+  g_object_get (self,
+        "resource", &resource,
+        "priority", &prio,
+        NULL);
+
+  if (priv->shared_statuses != NULL)
+    g_hash_table_destroy (priv->shared_statuses);
+
+  priv->shared_statuses = g_hash_table_new_full (
+      g_str_hash, g_str_equal, g_free, (GDestroyNotify) g_strfreev);
+
+  wocky_node_iter_init (&iter, query_node, NULL, NULL);
+  while (wocky_node_iter_next (&iter, &node))
+    {
+      if (g_strcmp0 (node->name, "status-list") == 0)
+        {
+          WockyNodeIter list_iter;
+          WockyNode *list_item;
+          GPtrArray *statuses;
+          const gchar *show = wocky_node_get_attribute (node, "show");
+
+          if (show == NULL)
+            continue;
+
+          statuses = g_ptr_array_new ();
+
+          wocky_node_iter_init (&list_iter, node, "status", NULL);
+          while (wocky_node_iter_next (&list_iter, &list_item))
+            g_ptr_array_add (statuses, g_strdup (list_item->content));
+
+          g_ptr_array_add (statuses, NULL);
+
+          g_hash_table_insert (priv->shared_statuses, g_strdup (show),
+              g_ptr_array_free (statuses, FALSE));
+        }
+      else if (g_strcmp0 (node->name, "status") == 0)
+        {
+          status_message = node->content;
+        }
+      else if (g_strcmp0 (node->name, "show") == 0)
+        {
+          dnd = g_strcmp0 (node->content, "dnd") == 0;
+        }
+      else if (g_strcmp0 (node->name, "invisible") == 0)
+        {
+          invisible = g_strcmp0 (wocky_node_get_attribute (node, "value"), "true") == 0;
+        }
+    }
+
+  priv->shared_status_compat =
+    g_strcmp0 (min_version, GOOGLE_SHARED_STATUS_VERSION) == 0;
+
+  if (invisible)
+    {
+      if (priv->shared_status_compat)
+        presence_id = GABBLE_PRESENCE_HIDDEN;
+      else
+        presence_id = GABBLE_PRESENCE_DND;
+    }
+  else if (dnd)
+    {
+      presence_id = GABBLE_PRESENCE_DND;
+    }
+  else
+    {
+      if (presence_id == GABBLE_PRESENCE_DND || presence_id == GABBLE_PRESENCE_HIDDEN)
+        presence_id = GABBLE_PRESENCE_AVAILABLE;
+    }
+
+  /* If we are connected, use the new shared status. If not, override with local */
+  if (base->status == TP_CONNECTION_STATUS_CONNECTED)
+    rv = gabble_presence_update (self->self_presence, resource, presence_id,
+        status_message, prio);
+  else
+    rv = TRUE;
+
+  g_free (resource);
+
+  return rv;
+}
+
+static gboolean
+iq_shared_status_changed_cb (WockyPorter *porter,
+    WockyStanza *stanza,
+    gpointer user_data)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (user_data);
+  WockyNode *query_node = wocky_node_get_child_ns (
+      wocky_stanza_get_top_node (stanza), "query",
+      NS_GOOGLE_SHARED_STATUS);
+  WockyStanza *result;
+
+  if (store_shared_statuses (self, query_node))
+    emit_presences_changed_for_self (self);
+
+  result = wocky_stanza_build_iq_result (stanza, NULL);
+
+  wocky_porter_send (porter, result);
+
+  g_object_unref (result);
+
+  return TRUE;
+}
+
 static LmHandlerResult
 iq_privacy_list_push_cb (LmMessageHandler *handler,
     LmConnection *connection,
@@ -575,8 +878,7 @@ iq_privacy_list_push_cb (LmMessageHandler *handler,
 
   result = lm_iq_message_make_result (message);
 
-  if (!lm_connection_send (conn->lmconn, result, NULL))
-      DEBUG ("sending push privacy list response failed");
+  wocky_porter_send (wocky_session_get_porter (conn->session), result);
 
   list_name = lm_message_node_get_attribute (list_node, "name");
 
@@ -618,6 +920,89 @@ iq_privacy_list_push_cb (LmMessageHandler *handler,
 * ...
 **********************************************************************/
 
+static void
+get_shared_status_cb  (GObject *source_object,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (source_object);
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (user_data);
+  WockyStanza *iq = NULL;
+  GError *error = NULL;
+
+  DEBUG (" ");
+
+  if (!conn_util_send_iq_finish (self, res, &iq, &error))
+    {
+      DEBUG ("Error getting shared status: %s", error->message);
+
+      g_simple_async_result_set_from_error (result, error);
+      g_error_free (error);
+    }
+  else
+    {
+      WockyNode *query_node = wocky_node_get_child_ns (wocky_stanza_get_top_node (iq),
+          "query", NS_GOOGLE_SHARED_STATUS);
+
+      if (query_node != NULL)
+        {
+          const gchar *max_shared = wocky_node_get_attribute (query_node,
+              "status-list-contents-max");
+
+          if (max_shared != NULL)
+            priv->max_shared_statuses = (gint) g_ascii_strtoll (max_shared, NULL, 10);
+          else
+            priv->max_shared_statuses = 5; /* Safe bet */
+
+          store_shared_statuses (self, query_node);
+        }
+      else
+        {
+          g_simple_async_result_set_error (result, CONN_PRESENCE_ERROR,
+              CONN_PRESENCE_ERROR_SET_SHARED_STATUS,
+              "Error retrieving shared status, received empty reply");
+        }
+
+      g_object_unref (iq);
+    }
+
+  g_simple_async_result_complete_in_idle (result);
+  g_object_unref (result);
+}
+
+static void
+get_shared_status_async (GabbleConnection *self,
+    GAsyncReadyCallback callback,
+    gpointer user_data)
+{
+  GSimpleAsyncResult *result = g_simple_async_result_new (G_OBJECT (self),
+      callback, user_data, get_shared_status_async);
+  WockyStanza *iq;
+
+  DEBUG (" ");
+
+  iq = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+      WOCKY_STANZA_SUB_TYPE_GET, NULL, conn_util_get_bare_self_jid (self),
+        '(', "query",
+          ':', NS_GOOGLE_SHARED_STATUS,
+          '@', "version", GOOGLE_SHARED_STATUS_VERSION,
+        ')',
+      NULL);
+
+  conn_util_send_iq_async (self, iq, NULL, get_shared_status_cb, result);
+
+  g_object_unref (iq);
+}
+
+static gboolean
+get_shared_status_finish (GabbleConnection *self,
+    GAsyncResult *result,
+    GError **error)
+{
+  wocky_implement_finish_void (self, get_shared_status_async);
+}
+
 static LmHandlerResult
 get_existing_privacy_lists_cb (GabbleConnection *conn,
     LmMessage *sent_msg,
@@ -625,25 +1010,31 @@ get_existing_privacy_lists_cb (GabbleConnection *conn,
     GObject *obj,
     gpointer user_data)
 {
+  GSimpleAsyncResult *result = G_SIMPLE_ASYNC_RESULT (user_data);
+  WockyNode *query_node = wocky_node_get_child_ns (
+      wocky_stanza_get_top_node (reply_msg), "query", NS_PRIVACY);
   GError *error = gabble_message_get_xmpp_error (reply_msg);
 
-  if (error)
+  if (error != NULL)
     {
       DEBUG ("Error getting privacy lists: %s", error->message);
 
-      g_simple_async_result_set_from_error (user_data, error);
+      g_simple_async_result_set_from_error (result, error);
       g_error_free (error);
     }
+  else if (query_node == NULL)
+    {
+      g_simple_async_result_set_error (result,
+          TP_ERRORS, TP_ERROR_NETWORK_ERROR,
+          "no <query/> node in 'list privacy lists' reply");
+    }
   else
     {
       GabbleConnectionPresencePrivate *priv = conn->presence_priv;
-      LmMessageNode *iq, *list_node;
+      LmMessageNode *list_node;
       WockyNodeIter iter;
       GabblePluginLoader *loader = gabble_plugin_loader_dup ();
 
-      iq = lm_message_get_node (reply_msg);
-      iq = lm_message_node_get_child_with_namespace (iq, "query", NS_PRIVACY);
-
       /* As we're called only once, privacy_statuses couldn't have been
        * already initialised. */
       g_assert (priv->privacy_statuses == NULL);
@@ -651,7 +1042,7 @@ get_existing_privacy_lists_cb (GabbleConnection *conn,
       priv->privacy_statuses = g_hash_table_new_full (
           g_str_hash, g_str_equal, g_free, g_free);
 
-      wocky_node_iter_init (&iter, iq, "list", NULL);
+      wocky_node_iter_init (&iter, query_node, "list", NULL);
       while (wocky_node_iter_next (&iter, &list_node))
         {
           const gchar *list_name = lm_message_node_get_attribute (list_node,
@@ -673,13 +1064,12 @@ get_existing_privacy_lists_cb (GabbleConnection *conn,
         }
     }
 
-  g_simple_async_result_complete_in_idle (user_data);
-  g_object_unref (user_data);
+  g_simple_async_result_complete_in_idle (result);
+  g_object_unref (result);
 
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
-
 static void
 get_existing_privacy_lists_async (GabbleConnection *self,
     GAsyncReadyCallback callback,
@@ -950,29 +1340,66 @@ privacy_lists_loaded_cb (GObject *source_object,
       priv->invisible_list_name =
           g_strdup (g_hash_table_lookup (priv->privacy_statuses, "hidden"));
 
-      priv->invisibility_method = INVISIBILITY_METHOD_PRIVACY;
+      if (priv->invisibility_method == INVISIBILITY_METHOD_NONE)
+        priv->invisibility_method = INVISIBILITY_METHOD_PRIVACY;
+    }
 
-      setup_invisible_privacy_list_async (self, initial_presence_setup_cb,
-          user_data);
+  if (priv->invisibility_method == INVISIBILITY_METHOD_PRIVACY)
+    setup_invisible_privacy_list_async (self, initial_presence_setup_cb,
+        user_data);
+  else
+    toggle_presence_visibility_async (self,
+        toggle_initial_presence_visibility_cb, user_data);
+}
+
+static void
+shared_status_setup_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  GError *error = NULL;
+  GabbleConnection *self = GABBLE_CONNECTION (source_object);
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+
+  if (get_shared_status_finish (self, result, &error))
+    {
+      WockyPorter *porter = wocky_session_get_porter (self->session);
+
+      priv->invisibility_method = INVISIBILITY_METHOD_SHARED_STATUS;
+
+      priv->iq_shared_status_cb = wocky_c2s_porter_register_handler_from_server (
+          WOCKY_C2S_PORTER (porter),
+          WOCKY_STANZA_TYPE_IQ, WOCKY_STANZA_SUB_TYPE_SET,
+          WOCKY_PORTER_HANDLER_PRIORITY_NORMAL,
+          iq_shared_status_changed_cb, self,
+          '(', "query",
+            ':', NS_GOOGLE_SHARED_STATUS,
+          ')', NULL);
     }
   else
     {
-      if (self->features & GABBLE_CONNECTION_FEATURES_INVISIBLE)
-          priv->invisibility_method = INVISIBILITY_METHOD_INVISIBLE_COMMAND;
-
-      toggle_presence_visibility_async (self,
-          toggle_initial_presence_visibility_cb, user_data);
+      DEBUG ("failed: %s", error->message);
+      g_error_free (error);
     }
+
+  get_existing_privacy_lists_async (self, privacy_lists_loaded_cb, user_data);
 }
 
 void
 conn_presence_set_initial_presence_async (GabbleConnection *self,
     GAsyncReadyCallback callback, gpointer user_data)
 {
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
   GSimpleAsyncResult *result = g_simple_async_result_new (G_OBJECT (self),
       callback, user_data, conn_presence_set_initial_presence_async);
 
-  get_existing_privacy_lists_async (self, privacy_lists_loaded_cb, result);
+  if (self->features & GABBLE_CONNECTION_FEATURES_INVISIBLE)
+    priv->invisibility_method = INVISIBILITY_METHOD_INVISIBLE_COMMAND;
+
+  if (self->features & GABBLE_CONNECTION_FEATURES_GOOGLE_SHARED_STATUS)
+    get_shared_status_async (self, shared_status_setup_cb, result);
+  else
+    get_existing_privacy_lists_async (self, privacy_lists_loaded_cb, result);
 }
 
 gboolean
@@ -980,7 +1407,34 @@ conn_presence_set_initial_presence_finish (GabbleConnection *self,
     GAsyncResult *result,
     GError **error)
 {
-  wocky_implement_finish_void (self, conn_presence_set_initial_presence_async)
+  wocky_implement_finish_void (self, conn_presence_set_initial_presence_async);
+}
+
+/**
+ * conn_presence_statuses:
+ *
+ * Used to retrieve the list of presence statuses supported by connections
+ * consisting of the basic statuses followed by any statuses defined by
+ * plugins.
+ *
+ * Returns: an array of #TpPresenceStatusSpec terminated by a 0 filled member
+ * The array is owned by te connection presence implementation and must not
+ * be altered or freed by anyone else.
+ **/
+const TpPresenceStatusSpec *
+conn_presence_statuses (void)
+{
+  if (gabble_statuses == NULL)
+    {
+      GabblePluginLoader *loader = gabble_plugin_loader_dup ();
+
+      gabble_statuses = gabble_plugin_loader_append_statuses (
+          loader, gabble_base_statuses);
+
+      g_object_unref (loader);
+    }
+
+  return gabble_statuses;
 }
 
 /**
@@ -1036,8 +1490,7 @@ conn_presence_visible_to (GabbleConnection *self,
   if (self->self_presence->status == GABBLE_PRESENCE_HIDDEN)
     return FALSE;
 
-  if ((gabble_roster_handle_get_subscription (self->roster, recipient)
-      & GABBLE_ROSTER_SUBSCRIPTION_FROM) == 0)
+  if (!gabble_roster_handle_gets_presence_from_us (self->roster, recipient))
     return FALSE;
 
   /* FIXME: other reasons they might not be able to see our presence? */
@@ -1070,6 +1523,10 @@ toggle_presence_visibility_async (GabbleConnection *self,
 
   switch (priv->invisibility_method)
     {
+      case INVISIBILITY_METHOD_SHARED_STATUS:
+        set_shared_status (self, result);
+        break;
+
       case INVISIBILITY_METHOD_INVISIBLE_COMMAND:
         set_xep0186_invisible (self, set_invisible, result);
         break;
@@ -1107,7 +1564,28 @@ toggle_presence_visibility_finish (
     GAsyncResult *result,
     GError **error)
 {
-  wocky_implement_finish_void (self, toggle_presence_visibility_async)
+  wocky_implement_finish_void (self, toggle_presence_visibility_async);
+}
+
+static void
+set_shared_status_presence_cb (GObject *source_object,
+    GAsyncResult *res,
+    gpointer user_data)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (source_object);
+  GError *error = NULL;
+
+  DEBUG (" ");
+  if (!set_shared_status_finish (self, res, &error))
+    {
+      DEBUG ("Error setting shared status %s",
+          error->message);
+
+      g_error_free (error);
+      error = NULL;
+    }
+
+  emit_presences_changed_for_self (self);
 }
 
 static void
@@ -1230,6 +1708,10 @@ set_own_status_cb (GObject *obj,
         {
           activate_current_privacy_list_async (conn, NULL, NULL);
         }
+      else if (priv->shared_statuses != NULL)
+        {
+          set_shared_status_async (conn, set_shared_status_presence_cb, NULL);
+        }
       else
         {
           retval = conn_presence_signal_own_presence (conn, NULL, error);
@@ -1279,9 +1761,9 @@ status_available_cb (GObject *obj, guint status)
     gabble_statuses[status].presence_type;
 
   /* This relies on the fact the first entries in the statuses table
-   * are from base_statuses. If index to the statuses table is outside
-   * the base_statuses table, the status is provided by a plugin. */
-  if (status >= G_N_ELEMENTS (base_statuses))
+   * are from gabble_base_statuses. If index to the statuses table is outside
+   * the gabble_base_statuses table, the status is provided by a plugin. */
+  if (status >= G_N_ELEMENTS (gabble_base_statuses))
     {
       /* At the moment, plugins can only implement statuses via privacy
        * lists, so any extra status should be backed by one. If it's not
@@ -1325,7 +1807,6 @@ conn_presence_get_type (GabblePresence *presence)
   return gabble_statuses[presence->status].presence_type;
 }
 
-
 /* We should update this when telepathy-glib supports setting
  * statuses at constructor time (see
  *   https://bugs.freedesktop.org/show_bug.cgi?id=12896 ).
@@ -1334,26 +1815,15 @@ conn_presence_get_type (GabblePresence *presence)
 void
 conn_presence_class_init (GabbleConnectionClass *klass)
 {
-  if (gabble_statuses == NULL)
-    {
-      GabblePluginLoader *loader = gabble_plugin_loader_dup ();
-
-      gabble_statuses = gabble_plugin_loader_append_statuses (
-          loader, base_statuses);
-
-      g_object_unref (loader);
-    }
-
   tp_presence_mixin_class_init ((GObjectClass *) klass,
       G_STRUCT_OFFSET (GabbleConnectionClass, presence_class),
       status_available_cb, construct_contact_statuses_cb,
-      set_own_status_cb, gabble_statuses);
+      set_own_status_cb, conn_presence_statuses ());
 
   tp_presence_mixin_simple_presence_init_dbus_properties (
     (GObjectClass *) klass);
 }
 
-
 void
 conn_presence_init (GabbleConnection *conn)
 {
@@ -1376,6 +1846,23 @@ conn_presence_init (GabbleConnection *conn)
       G_OBJECT (conn));
 }
 
+void
+conn_presence_dispose (GabbleConnection *self)
+{
+  GabbleConnectionPresencePrivate *priv = self->presence_priv;
+  WockyPorter *porter;
+
+  if (self->session == NULL)
+    return;
+
+  porter = wocky_session_get_porter (self->session);
+
+  if (priv->iq_shared_status_cb != 0)
+    {
+      wocky_porter_unregister_handler (porter, priv->iq_shared_status_cb);
+      priv->iq_shared_status_cb = 0;
+    }
+}
 
 void
 conn_presence_finalize (GabbleConnection *conn)
@@ -1387,6 +1874,9 @@ conn_presence_finalize (GabbleConnection *conn)
   if (priv->privacy_statuses != NULL)
       g_hash_table_destroy (priv->privacy_statuses);
 
+  if (priv->shared_statuses != NULL)
+      g_hash_table_destroy (priv->shared_statuses);
+
   if (priv->iq_list_push_cb != NULL)
     lm_message_handler_unref (priv->iq_list_push_cb);
 
