@@ -39,8 +39,6 @@
 #include "ft-manager.h"
 #include "jingle-factory.h"
 #include "muc-factory.h"
-#include "olpc-gadget-manager.h"
-#include "slacker.h"
 #include "types.h"
 
 G_BEGIN_DECLS
@@ -79,6 +77,8 @@ typedef enum
   GABBLE_CONNECTION_FEATURES_PEP = 1 << 4,
   GABBLE_CONNECTION_FEATURES_GOOGLE_MAIL_NOTIFY = 1 << 5,
   GABBLE_CONNECTION_FEATURES_INVISIBLE = 1 << 6,
+  GABBLE_CONNECTION_FEATURES_GOOGLE_SHARED_STATUS = 1 << 7,
+  GABBLE_CONNECTION_FEATURES_GOOGLE_QUEUE = 1 << 8,
 } GabbleConnectionFeatures;
 
 typedef struct _GabbleConnectionPrivate GabbleConnectionPrivate;
@@ -91,16 +91,6 @@ typedef LmHandlerResult (*GabbleConnectionMsgReplyFunc) (GabbleConnection *conn,
                                                          GObject *object,
                                                          gpointer user_data);
 
-/* must be in the same order as the list_handle_strings in
- * connection.c */
-typedef enum
-{
-  GABBLE_LIST_HANDLE_STORED = 1,
-  GABBLE_LIST_HANDLE_PUBLISH,
-  GABBLE_LIST_HANDLE_SUBSCRIBE,
-  GABBLE_LIST_HANDLE_DENY
-} GabbleListHandle;
-
 typedef enum {
     /* The JID could be a "global" JID, or a MUC room member. We'll assume
      * that it's a global JID (and remove the resource) unless we've seen
@@ -162,14 +152,6 @@ struct _GabbleConnection {
     GHashTable *olpc_invited_activities;
     GHashTable *olpc_current_act;
 
-    /* OLPC services */
-    const gchar *olpc_gadget_buddy;
-    const gchar *olpc_gadget_activity;
-    gboolean olpc_gadget_publish;
-
-    /* OLPC Gadget manager */
-    GabbleOlpcGadgetManager *olpc_gadget_manager;
-
     /* bytestream factory */
     GabbleBytestreamFactory *bytestream_factory;
 
@@ -206,9 +188,6 @@ struct _GabbleConnection {
     /* ContactInfo.SupportedFields, or NULL to use the generic one */
     GPtrArray *contact_info_fields;
 
-    GabbleSlacker *slacker;
-    guint slacker_inactivity_changed_id;
-
     GabbleConnectionPrivate *priv;
 };
 
@@ -224,7 +203,7 @@ typedef enum {
 
 gchar *gabble_connection_get_full_jid (GabbleConnection *conn);
 
-WockyPorter *gabble_connection_get_porter (GabbleConnection *conn);
+WockyPorter *gabble_connection_dup_porter (GabbleConnection *conn);
 
 gboolean _gabble_connection_set_properties_from_account (
     GabbleConnection *conn, const gchar *account, GError **error);
