@@ -108,6 +108,14 @@
  *
  */
 
+static gboolean test_mode = FALSE;
+
+void
+gtalk_file_collection_set_test_mode (void)
+{
+  test_mode = TRUE;
+}
+
 G_DEFINE_TYPE (GTalkFileCollection, gtalk_file_collection, G_TYPE_OBJECT);
 
 /* properties */
@@ -241,43 +249,23 @@ gtalk_file_collection_dispose (GObject *object)
   self->priv->dispose_has_run = TRUE;
 
   if (self->priv->jingle != NULL)
-    {
-      gabble_jingle_session_terminate (self->priv->jingle,
-          TP_CHANNEL_GROUP_CHANGE_REASON_NONE, NULL, NULL);
+    gabble_jingle_session_terminate (self->priv->jingle,
+        JINGLE_REASON_UNKNOWN, NULL, NULL);
 
-      /* the terminate could synchronously unref it and set it to NULL */
-      if (self->priv->jingle != NULL)
-        {
-          g_object_unref (self->priv->jingle);
-          self->priv->jingle = NULL;
-        }
-    }
+  tp_clear_object (&self->priv->jingle);
 
   set_current_channel (self, NULL);
 
-  if (self->priv->channels_reading != NULL)
-    {
-      g_hash_table_destroy (self->priv->channels_reading);
-      self->priv->channels_reading = NULL;
-    }
-
-  if (self->priv->channels_usable != NULL)
-    {
-      g_hash_table_destroy (self->priv->channels_usable);
-      self->priv->channels_usable = NULL;
-    }
-
-  if (self->priv->share_channels != NULL)
-    {
-      g_hash_table_destroy (self->priv->share_channels);
-      self->priv->share_channels = NULL;
-    }
+  tp_clear_pointer (&self->priv->channels_reading, g_hash_table_destroy);
+  tp_clear_pointer (&self->priv->channels_usable, g_hash_table_destroy);
+  tp_clear_pointer (&self->priv->share_channels, g_hash_table_destroy);
 
   for (i = self->priv->channels; i; i = i->next)
     {
       GabbleFileTransferChannel *channel = i->data;
       g_object_weak_unref (G_OBJECT (channel), channel_disposed, self);
     }
+
   g_list_free (self->priv->channels);
 
   g_free (self->priv->token);
@@ -484,7 +472,7 @@ jingle_session_state_changed_cb (GabbleJingleSession *session,
 static void
 jingle_session_terminated_cb (GabbleJingleSession *session,
                        gboolean local_terminator,
-                       TpChannelGroupChangeReason reason,
+                       JingleReason reason,
                        const gchar *text,
                        gpointer user_data)
 {
@@ -961,6 +949,9 @@ content_new_share_channel_cb (GabbleJingleContent *content, const gchar *name,
   DEBUG ("New Share channel %s was created and linked to id %d", name,
       share_channel_id);
 
+  if (test_mode)
+    g_object_set (agent, "upnp", FALSE, NULL);
+
   share_channel->agent = agent;
   share_channel->stream_id = stream_id;
   share_channel->component_id = NICE_COMPONENT_TYPE_RTP;
@@ -1037,16 +1028,8 @@ free_share_channel (gpointer data)
 
   DEBUG ("Freeing jingle Share channel");
 
-  if (share_channel->write_buffer != NULL)
-    {
-      g_free (share_channel->write_buffer);
-      share_channel->write_buffer = NULL;
-    }
-  if (share_channel->read_buffer != NULL)
-    {
-      g_free (share_channel->read_buffer);
-      share_channel->read_buffer = NULL;
-    }
+  tp_clear_pointer (&share_channel->write_buffer, g_free);
+  tp_clear_pointer (&share_channel->read_buffer, g_free);
   g_object_unref (share_channel->agent);
   g_slice_free (ShareChannel, share_channel);
 }
@@ -1742,7 +1725,7 @@ gtalk_file_collection_terminate (GTalkFileCollection *self,
              jingle session */
           self->priv->status = GTALK_FT_STATUS_TERMINATED;
           gabble_jingle_session_terminate (self->priv->jingle,
-              TP_CHANNEL_GROUP_CHANGE_REASON_NONE, NULL, NULL);
+              JINGLE_REASON_UNKNOWN, NULL, NULL);
           return;
         }
       return;
@@ -1783,7 +1766,7 @@ channel_disposed (gpointer data, GObject *object)
              jingle session */
           self->priv->status = GTALK_FT_STATUS_TERMINATED;
           gabble_jingle_session_terminate (self->priv->jingle,
-              TP_CHANNEL_GROUP_CHANGE_REASON_NONE, NULL, NULL);
+              JINGLE_REASON_UNKNOWN, NULL, NULL);
           return;
         }
     }
