@@ -104,8 +104,6 @@ gabble_im_channel_init (GabbleIMChannel *self)
   self->priv = priv;
 }
 
-#define NUM_SUPPORTED_MESSAGE_TYPES 3
-
 static void
 gabble_im_channel_constructed (GObject *obj)
 {
@@ -118,7 +116,7 @@ gabble_im_channel_constructed (GObject *obj)
       tp_base_connection_get_handles (base_conn, TP_HANDLE_TYPE_CONTACT);
   TpHandle target = tp_base_channel_get_target_handle (base);
 
-  TpChannelTextMessageType types[NUM_SUPPORTED_MESSAGE_TYPES] = {
+  TpChannelTextMessageType types[] = {
       TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
       TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION,
       TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE,
@@ -135,8 +133,7 @@ gabble_im_channel_constructed (GObject *obj)
 
   priv->peer_jid = g_strdup (tp_handle_inspect (contact_handles, target));
 
-  if (gabble_roster_handle_get_subscription (conn->roster, target)
-        & GABBLE_ROSTER_SUBSCRIPTION_FROM)
+  if (gabble_roster_handle_gets_presence_from_us (conn->roster, target))
     priv->send_nick = FALSE;
   else
     priv->send_nick = TRUE;
@@ -147,7 +144,7 @@ gabble_im_channel_constructed (GObject *obj)
       base_conn);
 
   tp_message_mixin_implement_sending (obj, _gabble_im_channel_send_message,
-      NUM_SUPPORTED_MESSAGE_TYPES, types, 0,
+      G_N_ELEMENTS (types), types, 0,
       TP_DELIVERY_REPORTING_SUPPORT_FLAG_RECEIVE_FAILURES,
       supported_content_types);
 }
@@ -155,6 +152,24 @@ gabble_im_channel_constructed (GObject *obj)
 static void gabble_im_channel_dispose (GObject *object);
 static void gabble_im_channel_finalize (GObject *object);
 
+static void
+gabble_im_channel_fill_immutable_properties (TpBaseChannel *chan,
+    GHashTable *properties)
+{
+  TpBaseChannelClass *cls = TP_BASE_CHANNEL_CLASS (
+      gabble_im_channel_parent_class);
+
+  cls->fill_immutable_properties (chan, properties);
+
+  tp_dbus_properties_mixin_fill_properties_hash (
+      G_OBJECT (chan), properties,
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "MessagePartSupportFlags",
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "DeliveryReportingSupport",
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "SupportedContentTypes",
+      TP_IFACE_CHANNEL_INTERFACE_MESSAGES, "MessageTypes",
+      NULL);
+}
+
 static void
 gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
 {
@@ -173,6 +188,8 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   base_class->interfaces = gabble_im_channel_interfaces;
   base_class->target_handle_type = TP_HANDLE_TYPE_CONTACT;
   base_class->close = gabble_im_channel_close;
+  base_class->fill_immutable_properties =
+    gabble_im_channel_fill_immutable_properties;
 
   tp_message_mixin_init_dbus_properties (object_class);
 }
@@ -234,20 +251,17 @@ gabble_im_channel_dispose (GObject *object)
   GabbleConnection *conn =
       GABBLE_CONNECTION (tp_base_channel_get_connection (base));
   TpHandle target = tp_base_channel_get_target_handle (base);
-  GabblePresence *presence;
-  GabbleRosterSubscription subscription;
 
   if (priv->dispose_has_run)
     return;
 
   priv->dispose_has_run = TRUE;
 
-  subscription = gabble_roster_handle_get_subscription (conn->roster, target);
-
-  presence = gabble_presence_cache_get (conn->presence_cache, target);
-
-  if ((GABBLE_ROSTER_SUBSCRIPTION_TO & subscription) == 0)
+  if (!gabble_roster_handle_sends_presence_to_us (conn->roster, target))
     {
+      GabblePresence *presence = gabble_presence_cache_get (
+          conn->presence_cache, target);
+
       if (NULL != presence)
         {
           presence->keep_unavailable = FALSE;
@@ -351,12 +365,7 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
         }
       else
         {
-          priv->chat_states_supported = CHAT_STATES_SUPPORTED;
-
-          tp_svc_channel_interface_chat_state_emit_chat_state_changed (
-              (TpSvcChannelInterfaceChatState *) chan,
-              tp_base_channel_get_target_handle (base_chan),
-              (TpChannelChatState) state);
+          _gabble_im_channel_state_receive (chan, state);
         }
     }
   else
@@ -370,14 +379,14 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
       priv->chat_states_supported = CHAT_STATES_UNKNOWN;
     }
 
-  msg = tp_message_new (base_conn, 2, 2);
+  msg = tp_cm_message_new (base_conn, 2);
 
   /* Header */
   if (type != TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL)
     tp_message_set_uint32 (msg, 0, "message-type", type);
 
   if (timestamp != 0)
-    tp_message_set_uint64 (msg, 0, "message-sent", timestamp);
+    tp_message_set_int64 (msg, 0, "message-sent", timestamp);
 
   /* Body */
   tp_message_set_string (msg, 1, "content-type", "text/plain");
@@ -385,28 +394,22 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
 
   if (send_error == GABBLE_TEXT_CHANNEL_SEND_NO_ERROR)
     {
-      tp_message_set_handle (msg, 0, "message-sender", TP_HANDLE_TYPE_CONTACT,
-          sender);
-      tp_message_set_uint64 (msg, 0, "message-received", time (NULL));
+      tp_cm_message_set_sender (msg, sender);
+      tp_message_set_int64 (msg, 0, "message-received", time (NULL));
 
-      /* We can't trust the id='' attribute set by the contact to be unique
-       * enough to be a message-token, so let's generate one locally.
-       */
-      tmp = gabble_generate_id ();
-      tp_message_set_string (msg, 0, "message-token", tmp);
-      g_free (tmp);
+      if (id != NULL)
+        tp_message_set_string (msg, 0, "message-token", id);
 
       tp_message_mixin_take_received (G_OBJECT (chan), msg);
     }
   else
     {
-      TpMessage *delivery_report = tp_message_new (base_conn, 1, 1);
+      TpMessage *delivery_report = tp_cm_message_new (base_conn, 1);
 
       tp_message_set_uint32 (delivery_report, 0, "message-type",
           TP_CHANNEL_TEXT_MESSAGE_TYPE_DELIVERY_REPORT);
-      tp_message_set_handle (delivery_report, 0, "message-sender",
-          TP_HANDLE_TYPE_CONTACT, sender);
-      tp_message_set_uint64 (delivery_report, 0, "message-received",
+      tp_cm_message_set_sender (delivery_report, sender);
+      tp_message_set_int64 (delivery_report, 0, "message-received",
           time (NULL));
 
       tmp = gabble_generate_id ();
@@ -422,14 +425,13 @@ _gabble_im_channel_receive (GabbleIMChannel *chan,
 
       /* We're getting a send error, so the original sender of the echoed
        * message must be us! */
-      tp_message_set_handle (msg, 0, "message-sender", TP_HANDLE_TYPE_CONTACT,
-          base_conn->self_handle);
+      tp_cm_message_set_sender (msg, base_conn->self_handle);
 
       /* Since this is a send error, we can trust the id on the message. */
       if (id != NULL)
         tp_message_set_string (msg, 0, "message-token", id);
 
-      tp_message_take_message (delivery_report, 0, "delivery-echo", msg);
+      tp_cm_message_take_message (delivery_report, 0, "delivery-echo", msg);
 
       tp_message_mixin_take_received (G_OBJECT (chan), delivery_report);
     }
