@@ -32,6 +32,7 @@
 
 #include "connection.h"
 #include "debug.h"
+#include "gabble-signals-marshal.h"
 #include "jingle-share.h"
 #include "jingle-media-rtp.h"
 #include "jingle-session.h"
@@ -47,6 +48,7 @@ G_DEFINE_TYPE(GabbleJingleFactory, gabble_jingle_factory, G_TYPE_OBJECT);
 enum
 {
     NEW_SESSION,
+    STUN_SERVER_CHANGED,
     LAST_SIGNAL
 };
 
@@ -208,6 +210,9 @@ stun_server_resolved_cb (GObject *resolver,
       g_free (self->priv->stun_server);
       self->priv->stun_server = stun_server;
       self->priv->stun_port = data->stun_port;
+
+      g_signal_emit (self, signals[STUN_SERVER_CHANGED], 0,
+          stun_server, data->stun_port);
     }
 
 out:
@@ -254,6 +259,22 @@ got_jingle_info_stanza (GabbleJingleFactory *fac,
   GabbleJingleFactoryPrivate *priv = fac->priv;
   LmMessageSubType sub_type;
   LmMessageNode *query_node, *node;
+  const gchar *from = wocky_stanza_get_from (message);
+
+  if (from != NULL)
+    {
+      TpBaseConnection *base_conn = TP_BASE_CONNECTION (priv->conn);
+      TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+          base_conn, TP_HANDLE_TYPE_CONTACT);
+      TpHandle sender = tp_handle_lookup (contact_repo, from, NULL, NULL);
+
+      if (sender != base_conn->self_handle)
+        {
+          DEBUG ("ignoring jingleinfo from '%s', not ourself nor the server",
+              from);
+          return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+        }
+    }
 
   query_node = lm_message_node_get_child_with_namespace (
       wocky_stanza_get_top_node (message), "query", NS_GOOGLE_JINGLE_INFO);
@@ -480,25 +501,14 @@ gabble_jingle_factory_dispose (GObject *object)
   DEBUG ("dispose called");
   priv->dispose_has_run = TRUE;
 
-  if (priv->soup != NULL)
-    {
-      g_object_unref (priv->soup);
-      priv->soup = NULL;
-    }
-
-  g_hash_table_destroy (priv->sessions);
-  priv->sessions = NULL;
-
-  g_hash_table_destroy (priv->content_types);
-  priv->content_types = NULL;
-
-  g_hash_table_destroy (priv->transports);
-  priv->transports = NULL;
-
-  g_free (fac->priv->stun_server);
-  g_free (fac->priv->fallback_stun_server);
-  g_free (fac->priv->relay_token);
-  g_free (fac->priv->relay_server);
+  tp_clear_object (&priv->soup);
+  tp_clear_pointer (&priv->sessions, g_hash_table_destroy);
+  tp_clear_pointer (&priv->content_types, g_hash_table_destroy);
+  tp_clear_pointer (&priv->transports, g_hash_table_destroy);
+  tp_clear_pointer (&priv->stun_server, g_free);
+  tp_clear_pointer (&priv->fallback_stun_server, g_free);
+  tp_clear_pointer (&priv->relay_token, g_free);
+  tp_clear_pointer (&priv->relay_server, g_free);
 
   if (G_OBJECT_CLASS (gabble_jingle_factory_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_jingle_factory_parent_class)->dispose (object);
@@ -596,6 +606,11 @@ gabble_jingle_factory_class_init (GabbleJingleFactoryClass *cls)
         G_TYPE_FROM_CLASS (cls), G_SIGNAL_RUN_LAST,
         0, NULL, NULL, g_cclosure_marshal_VOID__POINTER,
         G_TYPE_NONE, 1, G_TYPE_POINTER);
+
+  signals[STUN_SERVER_CHANGED] = g_signal_new ("stun-server-changed",
+        G_TYPE_FROM_CLASS (cls), G_SIGNAL_RUN_LAST,
+        0, NULL, NULL, gabble_marshal_VOID__STRING_UINT,
+      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_UINT);
 }
 
 static void
@@ -672,13 +687,10 @@ connection_status_changed_cb (GabbleConnection *conn,
               priv->jingle_cb, LM_MESSAGE_TYPE_IQ);
           lm_connection_unregister_message_handler (priv->conn->lmconn,
               priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ);
-
-          lm_message_handler_unref (priv->jingle_cb);
-          priv->jingle_cb = NULL;
-
-          lm_message_handler_unref (priv->jingle_info_cb);
-          priv->jingle_info_cb = NULL;
         }
+
+      tp_clear_pointer (&priv->jingle_cb, lm_message_handler_unref);
+      tp_clear_pointer (&priv->jingle_info_cb, lm_message_handler_unref);
       break;
     }
 }
@@ -824,8 +836,7 @@ REQUEST_ERROR:
   g_error_free (error);
 
   if (sess != NULL && new_session)
-    gabble_jingle_session_terminate (sess, TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
-        NULL, NULL);
+    gabble_jingle_session_terminate (sess, JINGLE_REASON_UNKNOWN, NULL, NULL);
 
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
