@@ -26,7 +26,7 @@
 
 G_BEGIN_DECLS
 
-typedef enum {
+typedef enum { /*< skip >*/
   /* not a jingle message */
   JINGLE_DIALECT_ERROR,
   /* old libjingle3 gtalk variant */
@@ -42,7 +42,7 @@ typedef enum {
 #define JINGLE_IS_GOOGLE_DIALECT(d)\
     ((d == JINGLE_DIALECT_GTALK3) || (d == JINGLE_DIALECT_GTALK4))
 
-typedef enum {
+typedef enum { /*< skip >*/
   JINGLE_STATE_INVALID = -1,
   JINGLE_STATE_PENDING_CREATED = 0,
   JINGLE_STATE_PENDING_INITIATE_SENT,
@@ -53,7 +53,7 @@ typedef enum {
   MAX_JINGLE_STATES
 } JingleState;
 
-typedef enum {
+typedef enum { /*< skip >*/
   JINGLE_ACTION_UNKNOWN,
   JINGLE_ACTION_CONTENT_ACCEPT,
   JINGLE_ACTION_CONTENT_ADD,
@@ -71,31 +71,53 @@ typedef enum {
   JINGLE_ACTION_INFO
 } JingleAction;
 
-typedef enum {
+typedef enum { /*< skip >*/
   JINGLE_CONTENT_SENDERS_NONE,
   JINGLE_CONTENT_SENDERS_INITIATOR,
   JINGLE_CONTENT_SENDERS_RESPONDER,
   JINGLE_CONTENT_SENDERS_BOTH
 } JingleContentSenders;
 
-typedef enum {
+typedef enum { /*< skip >*/
   JINGLE_TRANSPORT_UNKNOWN,
   JINGLE_TRANSPORT_GOOGLE_P2P,
   JINGLE_TRANSPORT_RAW_UDP,
   JINGLE_TRANSPORT_ICE_UDP,
 } JingleTransportType;
 
-typedef enum {
+typedef enum { /*< skip >*/
   JINGLE_TRANSPORT_PROTOCOL_UDP,
   JINGLE_TRANSPORT_PROTOCOL_TCP
 } JingleTransportProtocol;
 
-typedef enum {
+typedef enum { /*< skip >*/
   JINGLE_CANDIDATE_TYPE_LOCAL,
   JINGLE_CANDIDATE_TYPE_STUN,
   JINGLE_CANDIDATE_TYPE_RELAY
 } JingleCandidateType;
 
+typedef enum
+{
+  JINGLE_REASON_UNKNOWN,
+  JINGLE_REASON_ALTERNATIVE_SESSION,
+  JINGLE_REASON_BUSY,
+  JINGLE_REASON_CANCEL,
+  JINGLE_REASON_CONNECTIVITY_ERROR,
+  JINGLE_REASON_DECLINE,
+  JINGLE_REASON_EXPIRED,
+  JINGLE_REASON_FAILED_APPLICATION,
+  JINGLE_REASON_FAILED_TRANSPORT,
+  JINGLE_REASON_GENERAL_ERROR,
+  JINGLE_REASON_GONE,
+  JINGLE_REASON_INCOMPATIBLE_PARAMETERS,
+  JINGLE_REASON_MEDIA_ERROR,
+  JINGLE_REASON_SECURITY_ERROR,
+  JINGLE_REASON_SUCCESS,
+  JINGLE_REASON_TIMEOUT,
+  JINGLE_REASON_UNSUPPORTED_APPLICATIONS,
+  JINGLE_REASON_UNSUPPORTED_TRANSPORTS
+} JingleReason;
+
 typedef struct _GabbleJingleFactoryClass GabbleJingleFactoryClass;
 
 GType gabble_jingle_factory_get_type (void);
