@@ -36,11 +36,11 @@
 #include <dbus/dbus-glib.h>
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/intset.h>
+#include <wocky/wocky-caps-cache.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
 
 #include "capabilities.h"
-#include "caps-cache.h"
 #include "caps-channel-manager.h"
 #include "caps-hash.h"
 #include "conn-presence.h"
@@ -452,6 +452,57 @@ gabble_presence_cache_init (GabblePresenceCache *cache)
       (GDestroyNotify) g_hash_table_destroy);
 }
 
+static void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
+    const gchar *node, const gchar *ns);
+
+static void
+gabble_presence_cache_add_bundles (GabblePresenceCache *cache)
+{
+#define GOOGLE_BUNDLE(cap, features) \
+  gabble_presence_cache_add_bundle_caps (cache, \
+      "http://www.google.com/xmpp/client/caps#" cap, features); \
+  gabble_presence_cache_add_bundle_caps (cache, \
+      "http://talk.google.com/xmpp/client/caps#" cap, features);
+
+  /* Cache various bundle from the Google Talk clients as trusted.  Some old
+   * versions of Google Talk do not reply correctly to discovery requests.
+   * Plus, we know what Google's bundles mean, so it's a waste of time to disco
+   * them, particularly the ones for features we don't support. The desktop
+   * client doesn't currently have all of these, but it doesn't hurt to cache
+   * them anyway.
+   */
+  GOOGLE_BUNDLE ("voice-v1", NS_GOOGLE_FEAT_VOICE);
+  GOOGLE_BUNDLE ("video-v1", NS_GOOGLE_FEAT_VIDEO);
+
+  /* File transfer support */
+  GOOGLE_BUNDLE ("share-v1", NS_GOOGLE_FEAT_SHARE);
+
+  /* Not really sure what this ones is. */
+  GOOGLE_BUNDLE ("sms-v1", NULL);
+
+  /* TODO: remove this when we fix fd.o#22768. */
+  GOOGLE_BUNDLE ("pmuc-v1", NULL);
+
+  /* The camera-v1 bundle seems to mean "I have a camera plugged in". Not
+   * having it doesn't seem to affect anything, and we have no way of exposing
+   * that information anyway.
+   */
+  GOOGLE_BUNDLE ("camera-v1", NULL);
+
+#undef GOOGLE_BUNDLE
+
+  /* We should also cache the ext='' bundles Gabble advertises: older Gabbles
+   * advertise these and don't support hashed caps, and we shouldn't need to
+   * disco them.
+   */
+  gabble_presence_cache_add_bundle_caps (cache,
+      NS_GABBLE_CAPS "#" BUNDLE_VOICE_V1, NS_GOOGLE_FEAT_VOICE);
+  gabble_presence_cache_add_bundle_caps (cache,
+      NS_GABBLE_CAPS "#" BUNDLE_VIDEO_V1, NS_GOOGLE_FEAT_VIDEO);
+  gabble_presence_cache_add_bundle_caps (cache,
+      NS_GABBLE_CAPS "#" BUNDLE_SHARE_V1, NS_GOOGLE_FEAT_SHARE);
+}
+
 static GObject *
 gabble_presence_cache_constructor (GType type, guint n_props,
                                    GObjectConstructParam *props)
@@ -467,6 +518,8 @@ gabble_presence_cache_constructor (GType type, guint n_props,
   g_assert (priv->presence_handles != NULL);
   g_assert (priv->decloak_handles != NULL);
 
+  gabble_presence_cache_add_bundles ((GabblePresenceCache *) obj);
+
   priv->status_changed_cb = g_signal_connect (priv->conn, "status-changed",
       G_CALLBACK (gabble_presence_cache_status_changed_cb), obj);
 
@@ -492,30 +545,19 @@ gabble_presence_cache_dispose (GObject *object)
       priv->unsure_id = 0;
     }
 
-  g_hash_table_destroy (priv->decloak_requests);
-  priv->decloak_requests = NULL;
-  tp_handle_set_destroy (priv->decloak_handles);
-  priv->decloak_handles = NULL;
+  tp_clear_pointer (&priv->decloak_requests, g_hash_table_destroy);
+  tp_clear_pointer (&priv->decloak_handles, tp_handle_set_destroy);
 
   g_assert (priv->lm_message_cb == NULL);
   g_assert (priv->lm_presence_cb == NULL);
 
   g_signal_handler_disconnect (priv->conn, priv->status_changed_cb);
 
-  g_hash_table_destroy (priv->presence);
-  priv->presence = NULL;
-
-  g_hash_table_destroy (priv->capabilities);
-  priv->capabilities = NULL;
-
-  g_hash_table_destroy (priv->disco_pending);
-  priv->disco_pending = NULL;
-
-  tp_handle_set_destroy (priv->presence_handles);
-  priv->presence_handles = NULL;
-
-  g_hash_table_destroy (priv->location);
-  priv->location = NULL;
+  tp_clear_pointer (&priv->presence, g_hash_table_destroy);
+  tp_clear_pointer (&priv->capabilities, g_hash_table_destroy);
+  tp_clear_pointer (&priv->disco_pending, g_hash_table_destroy);
+  tp_clear_pointer (&priv->presence_handles, tp_handle_set_destroy);
+  tp_clear_pointer (&priv->location, g_hash_table_destroy);
 
   if (G_OBJECT_CLASS (gabble_presence_cache_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_presence_cache_parent_class)->dispose (object);
@@ -621,22 +663,15 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
 
     case TP_CONNECTION_STATUS_DISCONNECTED:
       if (priv->lm_message_cb != NULL)
-        {
-          lm_connection_unregister_message_handler (conn->lmconn,
-                                                    priv->lm_message_cb,
-                                                    LM_MESSAGE_TYPE_MESSAGE);
-          lm_message_handler_unref (priv->lm_message_cb);
-          priv->lm_message_cb = NULL;
-        }
+        lm_connection_unregister_message_handler (conn->lmconn,
+            priv->lm_message_cb, LM_MESSAGE_TYPE_MESSAGE);
 
       if (priv->lm_presence_cb != NULL)
-        {
-          lm_connection_unregister_message_handler (conn->lmconn,
-                                                    priv->lm_presence_cb,
-                                                    LM_MESSAGE_TYPE_PRESENCE);
-          lm_message_handler_unref (priv->lm_presence_cb);
-          priv->lm_presence_cb = NULL;
-        }
+        lm_connection_unregister_message_handler (conn->lmconn,
+            priv->lm_presence_cb, LM_MESSAGE_TYPE_PRESENCE);
+
+      tp_clear_pointer (&priv->lm_message_cb, lm_message_handler_unref);
+      tp_clear_pointer (&priv->lm_presence_cb, lm_message_handler_unref);
       break;
 
     default:
@@ -756,6 +791,12 @@ self_avatar_resolve_conflict (GabblePresenceCache *cache)
   GabblePresence *presence = priv->conn->self_presence;
   GError *error = NULL;
 
+  if (base_conn->status != TP_CONNECTION_STATUS_CONNECTED)
+    {
+      DEBUG ("no longer connected");
+      return;
+    }
+
   /* We don't want recursive image resetting
    *
    * FIXME: There is a race here: if the other resource sends us first the
@@ -867,7 +908,7 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
               "<NULL>" : presence->avatar_sha1);
           self_avatar_resolve_conflict (cache);
         }
-      else
+      else if (base_conn->status == TP_CONNECTION_STATUS_CONNECTED)
         {
           g_free (presence->avatar_sha1);
           presence->avatar_sha1 = g_strdup (sha1);
@@ -1113,6 +1154,107 @@ emit_capabilities_discovered (GabblePresenceCache *cache,
   g_signal_emit (cache, signals[CAPABILITIES_DISCOVERED], 0, handle);
 }
 
+static GPtrArray *
+client_types_from_message (TpHandle handle,
+    LmMessageNode *lm_node,
+    const gchar *resource)
+{
+  WockyNode *identity, *query_result = (WockyNode *) lm_node;
+  WockyNodeIter iter;
+  GPtrArray *array;
+
+  array = g_ptr_array_new_with_free_func (g_free);
+
+  /* Find all identity nodes in the result. */
+  wocky_node_iter_init (&iter, query_result,
+      "identity", NS_DISCO_INFO);
+  while (wocky_node_iter_next (&iter, &identity))
+    {
+      const gchar *category, *type;
+
+      category = wocky_node_get_attribute (identity, "category");
+      if (category == NULL)
+        continue;
+
+      /* Now get the client type */
+      type = wocky_node_get_attribute (identity, "type");
+      if (type == NULL)
+        continue;
+
+      /* So, turns out if you disco a specific resource of a gtalk
+      contact, the Google servers will reply with the identity node as
+      if you disco'd the bare jid, so will get something like:
+
+          <identity category='account' type='registered' name='Google Talk User Account'/>
+
+      which is just great. So, let's special case android phones as
+      their resources will start with "android" and let's just say
+      they're phones. */
+
+      if (!tp_strdiff (category, "account")
+          && g_str_has_prefix (resource, "android")
+          && !tp_strdiff (type, "registered"))
+        {
+          type = "phone";
+        }
+
+      DEBUG ("Got type for %u: %s", handle, type);
+
+      g_ptr_array_add (array, g_strdup (type));
+    }
+
+  if (array->len == 0)
+    {
+      DEBUG ("How very odd, we didn't get any client types");
+      g_ptr_array_unref (array);
+      return NULL;
+    }
+
+  return array;
+}
+
+static void
+_signal_presences_updated (GabblePresenceCache *cache,
+    TpHandle handle)
+{
+  GArray *handles;
+
+  handles = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+  g_array_append_val (handles, handle);
+  g_signal_emit (cache, signals[PRESENCES_UPDATED], 0, handles);
+  g_array_free (handles, TRUE);
+}
+
+static void
+process_client_types (
+    GabblePresenceCache *cache,
+    LmMessageNode *query_result,
+    TpHandle handle,
+    DiscoWaiter *waiter_self)
+{
+  GabblePresence *presence = gabble_presence_cache_get (cache, handle);
+  GPtrArray *client_types;
+
+  /* If the contact's gone offline since we sent the disco request, we have no
+   * presence to attach their freshly-discovered client types to.
+   */
+  if (presence == NULL)
+    return;
+
+  client_types = client_types_from_message (handle, query_result,
+      waiter_self->resource);
+
+  if (waiter_self->resource != NULL)
+    gabble_presence_update_client_types (presence, waiter_self->resource,
+        client_types);
+
+  if (client_types != NULL)
+    {
+      g_ptr_array_unref (client_types);
+      _signal_presences_updated (cache, handle);
+    }
+}
+
 static void
 _caps_disco_cb (GabbleDisco *disco,
                 GabbleDiscoRequest *request,
@@ -1179,6 +1321,9 @@ _caps_disco_cb (GabbleDisco *disco,
       goto OUT;
     }
 
+  process_client_types (cache, query_result, handle, waiter_self);
+
+  /* Now onto caps */
   cap_set = gabble_capability_set_new_from_stanza (query_result);
 
   /* Only 'sha-1' is mandatory to implement by XEP-0115. If the remote contact
@@ -1230,7 +1375,7 @@ _caps_disco_cb (GabbleDisco *disco,
   if (trust >= CAPABILITY_BUNDLE_ENOUGH_TRUST)
     {
       WockyNodeTree *query_node = wocky_node_tree_new_from_node (query_result);
-      GabbleCapsCache *caps_cache = gabble_caps_cache_dup_shared ();
+      WockyCapsCache *caps_cache = wocky_caps_cache_dup_shared ();
 
       if (DEBUGGING)
         {
@@ -1241,7 +1386,7 @@ _caps_disco_cb (GabbleDisco *disco,
         }
 
       /* Update external cache. */
-      gabble_caps_cache_insert (caps_cache, node, query_node);
+      wocky_caps_cache_insert (caps_cache, node, query_node);
       g_object_unref (caps_cache);
       g_object_unref (query_node);
 
@@ -1325,15 +1470,15 @@ _process_caps_uri (GabblePresenceCache *cache,
   GabbleCapabilitySet *cached_caps = NULL;
   GabblePresenceCachePrivate *priv;
   TpHandleRepoIface *contact_repo;
-  GabbleCapsCache *caps_cache;
+  WockyCapsCache *caps_cache;
 
   priv = cache->priv;
   contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
   info = capability_info_get (cache, uri);
 
-  caps_cache = gabble_caps_cache_dup_shared ();
-  cached_query_reply = gabble_caps_cache_lookup (caps_cache, uri);
+  caps_cache = wocky_caps_cache_dup_shared ();
+  cached_query_reply = wocky_caps_cache_lookup (caps_cache, uri);
 
   if (cached_query_reply != NULL)
     {
@@ -1349,8 +1494,6 @@ _process_caps_uri (GabblePresenceCache *cache,
               query_str);
           g_free (query_str);
         }
-
-      g_object_unref (cached_query_reply);
     }
 
   g_object_unref (caps_cache);
@@ -1368,7 +1511,28 @@ _process_caps_uri (GabblePresenceCache *cache,
           from);
 
       if (presence)
-        gabble_presence_set_capabilities (presence, resource, cap_set, serial);
+        {
+          gabble_presence_set_capabilities (
+              presence, resource, cap_set, serial);
+
+          /* We can only get this information from actual disco replies,
+           * so we depend on having this information from the caps cache. */
+          if (cached_query_reply != NULL)
+            {
+              WockyNode *query = wocky_node_tree_get_top_node (cached_query_reply);
+              GPtrArray *types = client_types_from_message (handle, query, resource);
+
+              if (resource != NULL)
+                gabble_presence_update_client_types (presence, resource, types);
+
+              if (types != NULL)
+                {
+                  g_ptr_array_unref (types);
+
+                  _signal_presences_updated (cache, handle);
+                }
+            }
+        }
       else
         DEBUG ("presence not found");
 
@@ -1401,7 +1565,7 @@ _process_caps_uri (GabblePresenceCache *cache,
           DEBUG ("updating serial for waiter (%s, %s) from %u to %u",
               from, uri, waiter->serial, serial);
           waiter->serial = serial;
-          return;
+          goto out;
         }
 
       waiter = disco_waiter_new (contact_repo, handle, resource,
@@ -1436,6 +1600,10 @@ _process_caps_uri (GabblePresenceCache *cache,
           waiter->disco_requested = TRUE;
         }
     }
+
+out:
+  if (cached_query_reply != NULL)
+    g_object_unref (cached_query_reply);
 }
 
 static void
@@ -1624,8 +1792,8 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
       break;
 
     case LM_MESSAGE_SUB_TYPE_UNAVAILABLE:
-      if (gabble_roster_handle_get_subscription (priv->conn->roster, handle)
-        & GABBLE_ROSTER_SUBSCRIPTION_FROM)
+      if (gabble_roster_handle_sends_presence_to_us (priv->conn->roster,
+            handle))
         presence_id = GABBLE_PRESENCE_OFFLINE;
       else
         presence_id = GABBLE_PRESENCE_UNKNOWN;
@@ -1858,13 +2026,7 @@ gabble_presence_cache_update (
   if (gabble_presence_cache_do_update (cache, handle, resource, presence_id,
       status_message, priority))
     {
-      GArray *handles;
-
-      handles = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
-
-      g_array_append_val (handles, handle);
-      g_signal_emit (cache, signals[PRESENCES_UPDATED], 0, handles);
-      g_array_free (handles, TRUE);
+      _signal_presences_updated (cache, handle);
     }
 
   gabble_presence_cache_maybe_remove (cache, handle);
@@ -1913,7 +2075,8 @@ gabble_presence_cache_update_many (
 
 }
 
-void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
+static void
+gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
     const gchar *node,
     const gchar *namespace)
 {
@@ -2298,3 +2461,30 @@ gabble_presence_cache_get_location (GabblePresenceCache *cache,
 
   return NULL;
 }
+
+gboolean
+gabble_presence_cache_disco_in_progress (GabblePresenceCache *cache,
+    TpHandle handle,
+    const gchar *resource)
+{
+  GabblePresenceCachePrivate *priv = cache->priv;
+  GList *l, *waiters;
+  gboolean out = FALSE;
+
+  waiters = g_hash_table_get_values (priv->disco_pending);
+
+  for (l = waiters; l != NULL; l = l->next)
+    {
+      DiscoWaiter *w = l->data;
+
+      if (w != NULL && w->handle == handle && !tp_strdiff (w->resource, resource))
+        {
+          out = TRUE;
+          break;
+        }
+    }
+
+  g_list_free (waiters);
+
+  return out;
+}
