@@ -271,8 +271,8 @@ static const gchar * const roomlist_channel_allowed_properties[] = {
 
 
 static void
-gabble_roomlist_manager_foreach_channel_class (TpChannelManager *manager,
-    TpChannelManagerChannelClassFunc func,
+gabble_roomlist_manager_type_foreach_channel_class (GType type,
+    TpChannelManagerTypeChannelClassFunc func,
     gpointer user_data)
 {
   GHashTable *table = g_hash_table_new_full (g_str_hash, g_str_equal,
@@ -287,7 +287,7 @@ gabble_roomlist_manager_foreach_channel_class (TpChannelManager *manager,
   g_value_set_uint (value, TP_HANDLE_TYPE_NONE);
   g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType", value);
 
-  func (manager, table, roomlist_channel_allowed_properties, user_data);
+  func (type, table, roomlist_channel_allowed_properties, user_data);
 
   g_hash_table_destroy (table);
 }
@@ -442,7 +442,8 @@ channel_manager_iface_init (gpointer g_iface,
   TpChannelManagerIface *iface = g_iface;
 
   iface->foreach_channel = gabble_roomlist_manager_foreach_channel;
-  iface->foreach_channel_class = gabble_roomlist_manager_foreach_channel_class;
+  iface->type_foreach_channel_class =
+      gabble_roomlist_manager_type_foreach_channel_class;
   iface->request_channel = gabble_roomlist_manager_request_channel;
   iface->create_channel = gabble_roomlist_manager_create_channel;
   iface->ensure_channel = gabble_roomlist_manager_ensure_channel;
