@@ -238,8 +238,7 @@ filter_cb (DBusConnection *conn,
       /* connection was disconnected */
       DEBUG ("connection was disconnected");
       dbus_connection_close (priv->dbus_conn);
-      dbus_connection_unref (priv->dbus_conn);
-      priv->dbus_conn = NULL;
+      tp_clear_pointer (&priv->dbus_conn, dbus_connection_unref);
       goto out;
     }
 
@@ -519,12 +518,7 @@ bytestream_state_changed_cb (GabbleBytestreamIface *bytestream,
 
   if (state == GABBLE_BYTESTREAM_STATE_CLOSED)
     {
-      if (priv->bytestream != NULL)
-        {
-          g_object_unref (priv->bytestream);
-          priv->bytestream = NULL;
-        }
-
+      tp_clear_object (&priv->bytestream);
       g_signal_emit (G_OBJECT (self), signals[CLOSED], 0);
     }
   else if (state == GABBLE_BYTESTREAM_STATE_OPEN)
@@ -557,17 +551,14 @@ gabble_tube_dbus_dispose (GObject *object)
     }
 
   if (priv->dbus_conn != NULL)
-    {
-      dbus_connection_close (priv->dbus_conn);
-      dbus_connection_unref (priv->dbus_conn);
-    }
+    dbus_connection_close (priv->dbus_conn);
+
+  tp_clear_pointer (&priv->dbus_conn, dbus_connection_unref);
 
   if (priv->dbus_srv != NULL)
-    {
-      dbus_server_disconnect (priv->dbus_srv);
-      dbus_server_unref (priv->dbus_srv);
-      priv->dbus_srv = NULL;
-    }
+    dbus_server_disconnect (priv->dbus_srv);
+
+  tp_clear_pointer (&priv->dbus_srv, dbus_server_unref);
 
   if (priv->socket_path != NULL)
     {
@@ -590,25 +581,18 @@ gabble_tube_dbus_dispose (GObject *object)
       priv->dbus_msg_queue_size = 0;
     }
 
-  g_free (priv->dbus_srv_addr);
-  priv->dbus_srv_addr = NULL;
-  g_free (priv->socket_path);
-  priv->socket_path = NULL;
-  g_free (priv->dbus_local_name);
-  priv->dbus_local_name = NULL;
+  tp_clear_pointer (&priv->dbus_srv_addr, g_free);
+  tp_clear_pointer (&priv->socket_path, g_free);
+  tp_clear_pointer (&priv->dbus_local_name, g_free);
 
-  if (priv->dbus_names)
+  if (priv->dbus_names != NULL)
     {
       g_hash_table_foreach (priv->dbus_names, unref_handle_foreach,
           contact_repo);
-      g_hash_table_destroy (priv->dbus_names);
     }
 
-  if (priv->dbus_name_to_handle)
-     {
-       g_hash_table_destroy (priv->dbus_name_to_handle);
-       priv->dbus_name_to_handle = NULL;
-     }
+  tp_clear_pointer (&priv->dbus_names, g_hash_table_destroy);
+  tp_clear_pointer (&priv->dbus_name_to_handle, g_hash_table_destroy);
 
   if (priv->reassembly_buffer)
     g_string_free (priv->reassembly_buffer, TRUE);
@@ -880,7 +864,7 @@ gabble_tube_dbus_constructor (GType type,
   GObject *obj;
   GabbleTubeDBus *self;
   GabbleTubeDBusPrivate *priv;
-  DBusGConnection *bus;
+  TpDBusDaemon *bus;
   TpHandleRepoIface *contact_repo;
   TpBaseConnection *base;
   guint access_control;
@@ -890,20 +874,18 @@ gabble_tube_dbus_constructor (GType type,
   self = GABBLE_TUBE_DBUS (obj);
 
   priv = GABBLE_TUBE_DBUS_GET_PRIVATE (self);
+  base = (TpBaseConnection *) priv->conn;
 
   /* Ref the initiator handle */
-  g_assert (priv->conn != NULL);
+  g_assert (base != NULL);
   g_assert (priv->initiator != 0);
-  contact_repo = tp_base_connection_get_handles
-      ((TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  contact_repo = tp_base_connection_get_handles (base, TP_HANDLE_TYPE_CONTACT);
   tp_handle_ref (contact_repo, priv->initiator);
 
-  bus = tp_get_bus ();
-  dbus_g_connection_register_g_object (bus, priv->object_path, obj);
-
   DEBUG ("Registering at '%s'", priv->object_path);
 
-  base = (TpBaseConnection *) priv->conn;
+  bus = tp_base_connection_get_dbus_daemon (base);
+  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
 
   priv->dbus_names = g_hash_table_new_full (g_direct_hash, g_direct_equal,
       NULL, g_free);
@@ -1180,7 +1162,7 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   signals[CLOSED] =
@@ -1189,7 +1171,7 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   signals[OFFERED] =
@@ -1198,7 +1180,7 @@ gabble_tube_dbus_class_init (GabbleTubeDBusClass *gabble_tube_dbus_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   gabble_tube_dbus_class->dbus_props_class.interfaces = prop_interfaces;
