@@ -1210,31 +1210,13 @@ gabble_tube_stream_dispose (GObject *object)
       g_string_free (path, TRUE);
     }
 
-  if (priv->transport_to_bytestream != NULL)
-    {
-      g_hash_table_destroy (priv->transport_to_bytestream);
-      priv->transport_to_bytestream = NULL;
-    }
-
-  if (priv->bytestream_to_transport != NULL)
-    {
-      g_hash_table_destroy (priv->bytestream_to_transport);
-      priv->bytestream_to_transport = NULL;
-    }
-
-  if (priv->transport_to_id != NULL)
-    {
-      g_hash_table_destroy (priv->transport_to_id);
-      priv->transport_to_id = NULL;
-    }
+  tp_clear_pointer (&priv->transport_to_bytestream, g_hash_table_destroy);
+  tp_clear_pointer (&priv->bytestream_to_transport, g_hash_table_destroy);
+  tp_clear_pointer (&priv->transport_to_id, g_hash_table_destroy);
 
   tp_handle_unref (contact_repo, priv->initiator);
 
-  if (priv->local_listener != NULL)
-    {
-      g_object_unref (priv->local_listener);
-      priv->local_listener = NULL;
-    }
+  tp_clear_object (&priv->local_listener);
 
   if (priv->muc != NULL)
     {
@@ -1508,7 +1490,7 @@ gabble_tube_stream_constructor (GType type,
 {
   GObject *obj;
   GabbleTubeStreamPrivate *priv;
-  DBusGConnection *bus;
+  TpDBusDaemon *bus;
   TpHandleRepoIface *contact_repo;
 
   obj = G_OBJECT_CLASS (gabble_tube_stream_parent_class)->
@@ -1548,8 +1530,8 @@ gabble_tube_stream_constructor (GType type,
       tp_external_group_mixin_init (obj, (GObject *) priv->muc);
     }
 
-  bus = tp_get_bus ();
-  dbus_g_connection_register_g_object (bus, priv->object_path, obj);
+  bus = tp_base_connection_get_dbus_daemon ((TpBaseConnection *) priv->conn);
+  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
 
   DEBUG ("Registering at '%s'", priv->object_path);
 
@@ -1734,7 +1716,7 @@ gabble_tube_stream_class_init (GabbleTubeStreamClass *gabble_tube_stream_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   signals[NEW_CONNECTION] =
@@ -1743,7 +1725,7 @@ gabble_tube_stream_class_init (GabbleTubeStreamClass *gabble_tube_stream_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__UINT,
+                  g_cclosure_marshal_VOID__UINT,
                   G_TYPE_NONE, 1, G_TYPE_UINT);
 
   signals[CLOSED] =
@@ -1752,7 +1734,7 @@ gabble_tube_stream_class_init (GabbleTubeStreamClass *gabble_tube_stream_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   signals[OFFERED] =
@@ -1761,7 +1743,7 @@ gabble_tube_stream_class_init (GabbleTubeStreamClass *gabble_tube_stream_class)
                   G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
                   0,
                   NULL, NULL,
-                  gabble_marshal_VOID__VOID,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   gabble_tube_stream_class->dbus_props_class.interfaces = prop_interfaces;
