@@ -15,5 +15,5 @@ def test(q, bus, conn, stream):
     q.expect('dbus-signal', signal='StatusChanged', args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
 
 if __name__ == '__main__':
-    exec_test(test)
+    exec_test(test, do_connect=False)
 
