@@ -5,7 +5,7 @@ import time
 import datetime
 import os
 
-from servicetest import EventPattern, assertEquals
+from servicetest import EventPattern, assertEquals, call_async
 from gabbletest import exec_test, sync_stream, make_result_iq
 import ns
 from bytestream import create_from_si_offer, announce_socks5_proxy
@@ -37,6 +37,8 @@ class File(object):
 
         self.offset = 0
 
+        self.uri = 'file:///tmp/%s' % self.name
+
     def compute_hash(self, hash_type):
         assert hash_type == cs.FILE_HASH_TYPE_MD5
         self.hash_type = hash_type
@@ -65,11 +67,7 @@ class FileTransferTest(object):
                     [cs.SOCKET_ACCESS_CONTROL_LOCALHOST])
 
     def connect(self):
-        self.conn.Connect()
-
-        _, vcard_event, roster_event, disco_event = self.q.expect_many(
-            EventPattern('dbus-signal', signal='StatusChanged',
-                args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
+        vcard_event, roster_event, disco_event = self.q.expect_many(
             EventPattern('stream-iq', to=None, query_ns='vcard-temp',
                 query_name='vCard'),
             EventPattern('stream-iq', query_ns=ns.ROSTER),
@@ -156,7 +154,8 @@ class ReceiveFileTest(FileTransferTest):
         FileTransferTest.__init__(self, bytestream_cls, file, address_type, access_control, access_control_param)
 
         self._actions = [self.connect, self.announce_contact,
-            self.send_ft_offer_iq, self.check_new_channel, self.create_ft_channel, self.accept_file,
+            self.send_ft_offer_iq, self.check_new_channel, self.create_ft_channel,
+            self.set_uri, self.accept_file,
             self.receive_file, self.close_channel, self.done]
 
     def send_ft_offer_iq(self):
@@ -224,6 +223,30 @@ class ReceiveFileTest(FileTransferTest):
 
         self.ft_path = path
 
+    def set_uri(self):
+        ft_props = dbus.Interface(self.ft_channel, cs.PROPERTIES_IFACE)
+
+        # URI is not set yet
+        uri = ft_props.Get(cs.CHANNEL_TYPE_FILE_TRANSFER, 'URI')
+        assertEquals('', uri)
+
+        # Setting URI
+        call_async(self.q, ft_props, 'Set',
+            cs.CHANNEL_TYPE_FILE_TRANSFER, 'URI', self.file.uri)
+
+        self.q.expect('dbus-signal', signal='URIDefined', args=[self.file.uri])
+
+        self.q.expect('dbus-return', method='Set')
+
+        # Check it has the right value now
+        uri = ft_props.Get(cs.CHANNEL_TYPE_FILE_TRANSFER, 'URI')
+        assertEquals(self.file.uri, uri)
+
+        # We can't change it once it has been set
+        call_async(self.q, ft_props, 'Set',
+            cs.CHANNEL_TYPE_FILE_TRANSFER, 'URI', 'badger://snake')
+        self.q.expect('dbus-error', method='Set', name=cs.INVALID_ARGUMENT)
+
     def accept_file(self):
         try:
             self.address = self.ft_channel.AcceptFile(self.address_type,
@@ -325,7 +348,7 @@ class SendFileTest(FileTransferTest):
         assert ({cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_FILE_TRANSFER,
                  cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT},
                 [cs.FT_CONTENT_HASH_TYPE, cs.TARGET_HANDLE, cs.TARGET_ID, cs.FT_CONTENT_TYPE,
-                 cs.FT_FILENAME, cs.FT_SIZE, cs.FT_CONTENT_HASH, cs.FT_DESCRIPTION, cs.FT_DATE]
+                 cs.FT_FILENAME, cs.FT_SIZE, cs.FT_CONTENT_HASH, cs.FT_DESCRIPTION, cs.FT_DATE, cs.FT_URI]
              ) in properties.get('RequestableChannelClasses'),\
                      properties['RequestableChannelClasses']
 
@@ -334,15 +357,15 @@ class SendFileTest(FileTransferTest):
                  cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
                  cs.FT_CONTENT_HASH_TYPE: cs.FILE_HASH_TYPE_MD5},
                 [cs.TARGET_HANDLE, cs.TARGET_ID, cs.FT_CONTENT_TYPE, cs.FT_FILENAME,
-                 cs.FT_SIZE, cs.FT_CONTENT_HASH, cs.FT_DESCRIPTION, cs.FT_DATE]
+                 cs.FT_SIZE, cs.FT_CONTENT_HASH, cs.FT_DESCRIPTION, cs.FT_DATE,
+                 cs.FT_URI]
              ) in properties.get('RequestableChannelClasses'),\
                      properties['RequestableChannelClasses']
 
-    def request_ft_channel(self):
+    def request_ft_channel(self, uri=True):
         requests_iface = dbus.Interface(self.conn, cs.CONN_IFACE_REQUESTS)
 
-        self.ft_path, props = requests_iface.CreateChannel({
-            cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_FILE_TRANSFER,
+        request = { cs.CHANNEL_TYPE: cs.CHANNEL_TYPE_FILE_TRANSFER,
             cs.TARGET_HANDLE_TYPE: cs.HT_CONTACT,
             cs.TARGET_HANDLE: self.handle,
             cs.FT_CONTENT_TYPE: self.file.content_type,
@@ -352,30 +375,38 @@ class SendFileTest(FileTransferTest):
             cs.FT_CONTENT_HASH: self.file.hash,
             cs.FT_DESCRIPTION: self.file.description,
             cs.FT_DATE:  self.file.date,
-            cs.FT_INITIAL_OFFSET: 0,
-            })
+            cs.FT_INITIAL_OFFSET: 0 }
+
+        if uri:
+            request[cs.FT_URI] = self.file.uri
+
+        self.ft_path, props = requests_iface.CreateChannel(request)
 
         # org.freedesktop.Telepathy.Channel D-Bus properties
-        assert props[cs.CHANNEL_TYPE] == cs.CHANNEL_TYPE_FILE_TRANSFER
-        assert props[cs.INTERFACES] == []
-        assert props[cs.TARGET_HANDLE] == self.handle
-        assert props[cs.TARGET_ID] == self.contact_name
-        assert props[cs.TARGET_HANDLE_TYPE] == cs.HT_CONTACT
-        assert props[cs.REQUESTED] == True
-        assert props[cs.INITIATOR_HANDLE] == self.self_handle
-        assert props[cs.INITIATOR_ID] == self.self_handle_name
+        assertEquals(cs.CHANNEL_TYPE_FILE_TRANSFER, props[cs.CHANNEL_TYPE])
+        assertEquals([], props[cs.INTERFACES])
+        assertEquals(self.handle, props[cs.TARGET_HANDLE])
+        assertEquals(self.contact_name, props[cs.TARGET_ID])
+        assertEquals(cs.HT_CONTACT, props[cs.TARGET_HANDLE_TYPE])
+        assert props[cs.REQUESTED]
+        assertEquals(self.self_handle, props[cs.INITIATOR_HANDLE])
+        assertEquals(self.self_handle_name, props[cs.INITIATOR_ID])
 
         # org.freedesktop.Telepathy.Channel.Type.FileTransfer D-Bus properties
-        assert props[cs.FT_STATE] == cs.FT_STATE_PENDING
-        assert props[cs.FT_CONTENT_TYPE] == self.file.content_type
-        assert props[cs.FT_FILENAME] == self.file.name
-        assert props[cs.FT_SIZE] == self.file.size
-        assert props[cs.FT_CONTENT_HASH_TYPE] == self.file.hash_type
-        assert props[cs.FT_CONTENT_HASH] == self.file.hash
-        assert props[cs.FT_DESCRIPTION] == self.file.description
-        assert props[cs.FT_DATE] == self.file.date
-        assert props[cs.FT_TRANSFERRED_BYTES] == 0
-        assert props[cs.FT_INITIAL_OFFSET] == 0
+        assertEquals(cs.FT_STATE_PENDING, props[cs.FT_STATE])
+        assertEquals(self.file.content_type, props[cs.FT_CONTENT_TYPE])
+        assertEquals(self.file.name, props[cs.FT_FILENAME])
+        assertEquals(self.file.size, props[cs.FT_SIZE])
+        assertEquals(self.file.hash_type, props[cs.FT_CONTENT_HASH_TYPE])
+        assertEquals(self.file.hash, props[cs.FT_CONTENT_HASH])
+        assertEquals(self.file.description, props[cs.FT_DESCRIPTION])
+        assertEquals(self.file.date, props[cs.FT_DATE])
+        assertEquals(0, props[cs.FT_TRANSFERRED_BYTES])
+        assertEquals(0, props[cs.FT_INITIAL_OFFSET])
+        if uri:
+            assertEquals(self.file.uri, props[cs.FT_URI])
+        else:
+            assertEquals('', props[cs.FT_URI])
 
         self.check_platform_socket_types(props[cs.FT_AVAILABLE_SOCKET_TYPES])
 
@@ -493,7 +524,7 @@ def platform_impls():
 
     return impls
 
-def exec_file_transfer_test(test_cls):
+def exec_file_transfer_test(test_cls, one_run=False):
     for bytestream_cls  in [
             bytestream.BytestreamIBBMsg,
             bytestream.BytestreamS5B,
@@ -511,3 +542,6 @@ def exec_file_transfer_test(test_cls):
             file.offset = 5
             test = test_cls(bytestream_cls, file, addr_type, access_control, access_control_param)
             exec_test(test.test)
+
+            if one_run:
+                return
