@@ -284,11 +284,6 @@ def rccs(q, bus, conn, stream):
     Tests that the connection's RequestableChannelClasses for StreamedMedia are
     sane.
     """
-    conn.Connect()
-
-    q.expect('dbus-signal', signal='StatusChanged',
-        args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
-
     rccs = conn.Properties.Get(cs.CONN_IFACE_REQUESTS,
         'RequestableChannelClasses')
 
@@ -326,6 +321,7 @@ def rccs(q, bus, conn, stream):
         expected_allowed = [
             cs.TARGET_ID, cs.TARGET_HANDLE,
             cs.CALL_INITIAL_VIDEO, cs.CALL_INITIAL_AUDIO,
+            cs.CALL_INITIAL_VIDEO_NAME, cs.CALL_INITIAL_AUDIO_NAME,
             cs.CALL_MUTABLE_CONTENTS
         ]
 
