@@ -19,17 +19,6 @@ def test(q, bus, conn, stream, channel_type):
     # Make gabble think this is a different client
     jt2.remote_caps['node'] = 'http://example.com/fake-client1'
 
-    conn.Connect()
-
-    q.expect('dbus-signal', signal='StatusChanged',
-            args=[cs.CONN_STATUS_CONNECTING, cs.CSR_REQUESTED])
-
-    q.expect('stream-authenticated')
-    q.expect('dbus-signal', signal='PresenceUpdate',
-        args=[{1L: (0L, {u'available': {}})}])
-    q.expect('dbus-signal', signal='StatusChanged',
-            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
-
     run_test(q, bus, conn, stream, jt, True, channel_type)
     run_test(q, bus, conn, stream, jt2, False, channel_type)
 
