@@ -11,10 +11,7 @@ import constants as cs
 import ns
 
 def test(q, bus, conn, stream):
-    conn.Connect()
-    _, event, event2 = q.expect_many(
-        EventPattern('dbus-signal', signal='StatusChanged',
-            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED]),
+    event, event2 = q.expect_many(
         EventPattern('stream-iq', to=None, query_ns='vcard-temp',
             query_name='vCard'),
         EventPattern('stream-iq', query_ns=ns.ROSTER))
@@ -39,12 +36,14 @@ def test(q, bus, conn, stream):
     group_iface = dbus.Interface(chan, cs.CHANNEL_IFACE_GROUP)
     assert group_iface.GetMembers() == []
     handle = conn.RequestHandles(1, ['bob@foo.com'])[0]
-    group_iface.AddMembers([handle], '')
+    call_async(q, group_iface, 'AddMembers', [handle], '')
 
     event = q.expect('stream-iq', iq_type='set', query_ns=ns.ROSTER)
     item = event.query.firstChildElement()
 
     acknowledge_iq(stream, event.stanza)
+    # FIXME: when we depend on a new enough tp-glib, expect AddMembers
+    # to return here
 
     call_async(q, conn.Aliasing, 'RequestAliases', [handle])
 
