@@ -126,4 +126,4 @@ def test(q, bus, conn, stream):
     chan.Close()
 
 if __name__ == '__main__':
-    exec_test(test)
+    exec_test(test, do_connect=False)
