@@ -10,11 +10,6 @@ import constants as cs
 from muctubeutil import get_muc_tubes_channel
 
 def test(q, bus, conn, stream):
-    conn.Connect()
-
-    _ = q.expect('dbus-signal', signal='StatusChanged',
-            args=[cs.CONN_STATUS_CONNECTED, cs.CSR_REQUESTED])
-
     handle, tubes_chan, tubes_iface = get_muc_tubes_channel(q, bus, conn,
         stream, 'chat@conf.localhost')
 
